# -*- coding: utf-8 -*-
"""
Created on Thu Oct 23 17:02:32 2014

@author: kuehn
"""
import numpy as np
import os
#import parameters_ginzburg_asymm_LFP_new_noisy_2015_7_6_6_31_23 as params
import parameters_ginzburg_asymm_LFP_new_noisy_mult_theta as params

from solve_ODE_num_m_and_c_two_pop_new_noisy_mult_theta import integrate_m_c_ode
from compute_corr import compute

from ginzburg_theory_osc_input import ginzburg
#from plot_ginzburg_osc_input_paper_figures import plot_freq_dependence_mean_corr

from scipy import signal

E = 0
I = 1
X = 2

EE = 0
EI = 1
II = 2
EX = 3
IX = 4

E1 = 0
E2 = 1
I1 = 2
I2 = 3

def loaddata(dp,h0, omega,T,suffix_general):
    """Loads raw data produced by the NEST simulation and produces four arrays 
    containing  the mean activity the populations E1, E2, I1,I2 for all
    time points."""
    print 'In loaddata!'
    print suffix_general
    print 'omega=', omega
    print 'T=', T
    print 'h0=', h0
    
    files = os.listdir(dp)

    exc1_file = [f for f in files if (f.find('exc1')!=-1 
                and f.find('omega=' + str(omega) + '_')!=-1 
                and f.find('T=' + str(T) + '_')!=-1 
                and f.find('h0=' + str(h0) + '_')!=-1 
                and f.find(suffix_general)!=-1 
                and os.stat(dp + f).st_size != 0)]
    exc2_file = [f for f in files if (f.find('exc2')!=-1 
                and f.find('omega=' + str(omega) + '_')!=-1 
                and f.find('T=' + str(T) + '_')!=-1 
                and f.find('h0=' + str(h0) + '_')!=-1 
                and f.find(suffix_general)!=-1 
                and os.stat(dp + f).st_size != 0)]
    inh1_file = [f for f in files if (f.find('inh1')!=-1 
                and f.find('omega=' + str(omega) + '_')!=-1 
                and f.find('T=' + str(T) + '_')!=-1 
                and f.find('h0=' + str(h0) + '_')!=-1 
                and f.find(suffix_general)!=-1 
                and os.stat(dp + f).st_size != 0)]
    inh2_file = [f for f in files if (f.find('inh2')!=-1 
                and f.find('omega=' + str(omega) + '_')!=-1 
                and f.find('T=' + str(T) + '_')!=-1 
                and f.find('h0=' + str(h0) + '_')!=-1 
                and f.find(suffix_general)!=-1 
                and os.stat(dp + f).st_size != 0)]
    
    if(len(exc1_file)>0):        
        print 'str(omega)=', str(omega)        
        print 'str(T)=', str(T)         
        print 'datapath: ' + dp
        print 'exc1_file: ' + exc1_file[0]
        print 'exc2_file: ' + exc2_file[0]
        print 'inh1_file: ' + inh1_file[0]
        print 'inh2_file: ' + inh2_file[0]
        m_exc1 = np.loadtxt(dp + exc1_file[0])
        m_exc2 = np.loadtxt(dp + exc2_file[0])
        m_inh1 = np.loadtxt(dp + inh1_file[0])
        m_inh2 = np.loadtxt(dp + inh2_file[0])  
        return m_exc1, m_exc2, m_inh1, m_inh2
    else:
        return (np.zeros(1000, dtype=float), np.zeros(1000, dtype=float), 
                np.zeros(1000, dtype=float), np.zeros(1000, dtype=float))


def ft_mean_act(m_exc1, m_exc2, m_inh1, m_inh2, N_one_period):
    """Calculates the Fourier transform of the time-dependent mean activity
    with substracted mean."""
    
    N_transient = (int(int(10.*params.tau_m/params.dt_samp)/N_one_period)+1) * N_one_period    
    N_ges = len(m_exc1)
    N_number_periods = int((N_ges-N_transient)/N_one_period)
    N_eff = N_number_periods * N_one_period
   
    m_eff = np.zeros((4,N_eff), dtype = float)     
    m_eff[E1,:] = m_exc1[N_transient:N_eff+N_transient]
    m_eff[E2,:] = m_exc2[N_transient:N_eff+N_transient]
    m_eff[I1,:] = m_inh1[N_transient:N_eff+N_transient]
    m_eff[I2,:] = m_inh2[N_transient:N_eff+N_transient]
    
    m_mean   = np.zeros((4,N_eff), dtype = float)
    for ii in range(0,4):    
        m_mean[ii,:] = np.mean(m_eff[ii])
    
    m_freq = np.fft.fft(m_eff-m_mean, axis = 1)/N_eff  # In numpy.fft, fft is defined without normalization. 
    freq = np.fft.fftfreq(N_eff, d=params.dt_samp)*1000
    
    f_welch, m_psp_welch = signal.welch(m_eff - m_mean, 1/params.dt_samp, nperseg=1024, axis = 1)
    
    return (freq, m_freq[E1,:], m_freq[E2,:], m_freq[I1,:], m_freq[I2,:], 
            m_mean[E1,:], m_mean[E2,:], m_mean[I1,:], m_mean[I2,:], f_welch, m_psp_welch)
  

def angle_original(angle_trans):
    """Corrects the phase shift due to the choice to take the imaginary value
    instead of the real one for an array of complex numbers."""
    result = np.zeros(len(angle_trans), dtype=float)
    result += ((angle_trans > -np.pi) & (angle_trans <   np.pi/2)) * (angle_trans + np.pi/2)
    result += ((angle_trans >= np.pi/2) & (angle_trans < np.pi))   * (angle_trans - 3*np.pi/2)   
    return result     
    
###############################################################################
### Calculations ##############################################################
###############################################################################
def evaluate(datafolder, h0, omega, n_per_period, T_total, suffix_general):
    """Calls the loaddata-function and calculates the Fourier coefficients
    up to order two for the mean activity and the correlations and also gives
    out the time course of mean activities and correlations averaged
    over periods and, for the correlations, also its Fourier transformation."""
    M_0 = np.zeros(4, dtype=complex)
    M_1 = np.zeros(4, dtype=complex)
    M_2 = np.zeros(4, dtype=complex)

    ##################################################
    ### Read out and analyze nest results: ###########
    ##################################################
    omega_kHz = omega/1000.
    m_exc1, m_exc2, m_inh1, m_inh2 = loaddata(datafolder, h0, omega, T_total, 
                                              suffix_general)

    compute_c = compute(m_exc1, m_exc2, m_inh1, m_inh2, params.tau_m, params.dt_samp)
    #compute_c contains all functions calculating time-dependent zero-time-lag
    #correlations for the simulated data    
    C_0 = compute_c.const_corr(n_per_period)             
    C_1 = np.zeros(3, dtype=complex)
    C_2 = np.zeros(3, dtype=complex)
        
    print 'len(m_exc1)=', len(m_exc1)
    print 'n_per_period=', n_per_period        
    m_exc1 = m_exc1/(params.N_E*params.frac_rec)
    m_exc2 = m_exc2/(params.N_E*params.frac_rec)
    m_inh1 = m_inh1/(params.N_I*params.frac_rec)
    m_inh2 = m_inh2/(params.N_I*params.frac_rec)
           
    (freq, me1_f, me2_f, mi1_f, mi2_f, me1_m, me2_m, mi1_m, mi2_m, f_welch, 
     m_psp_welch) = ft_mean_act(m_exc1, m_exc2, m_inh1, m_inh2, n_per_period)

    freq_corr, c_quer_bare_fs, c_naiv_fs = compute_c.Fourier_corr_quer(n_per_period, 1) #fs means "frequency space"    
            
    mean_per, c_quer = compute_c.compute_corr_ts_directly(n_per_period)

    t_list = np.arange(0,n_per_period) * params.dt_samp        
    for ii in range(0,3):
        C_2[ii] = np.mean(c_quer[ii,:] * np.exp(-2j * omega_kHz * t_list))
        C_1[ii] = np.mean(c_quer[ii,:] * np.exp(-1j * omega_kHz * t_list))
       
    M_0[E1] = me1_m[0]
    M_0[E2] = me2_m[0]
    M_0[I1] = mi1_m[0]
    M_0[I2] = mi2_m[0]
        
    M_1[E1] = me1_f[len(me1_f)/n_per_period]
    M_1[E2] = me2_f[len(me1_f)/n_per_period]
    M_1[I1] = mi1_f[len(me1_f)/n_per_period]
    M_1[I2] = mi2_f[len(me1_f)/n_per_period]
    
    M_2[E1] = me1_f[2.*len(me1_f)/n_per_period]
    M_2[E2] = me2_f[2.*len(me1_f)/n_per_period]
    M_2[I1] = mi1_f[2.*len(me1_f)/n_per_period]
    M_2[I2] = mi2_f[2.*len(me1_f)/n_per_period]
        
    m_time_dep = np.zeros((2,mean_per.shape[0]),dtype=float)
    m_time_dep[E,:] = 0.5*(mean_per[:,E1] + mean_per[:,E2])
    m_time_dep[I,:] = 0.5*(mean_per[:,I1] + mean_per[:,I2])        
    c_time_dep = c_quer[EE:EX,:]
    c_freq_dep_full = c_quer_bare_fs - c_naiv_fs
             
    return (M_0, M_1, M_2, C_0, C_1, C_1, C_1, C_2, 
            m_time_dep, c_time_dep, c_freq_dep_full, 
            f_welch, m_psp_welch, freq, me1_f, me2_f, mi1_f, mi2_f)

    
def write(foldername,filename, h0, omega, M_0_sim, M_1_sim, M_2_sim, 
          C_0_sim, C_1_sim, C_2_sim, 
          mean_t, c_t, c_f):
    """Writes the data evaluated from the simulation into a npz-file."""
    stream_out = {}
    stream_out['h0'] = h0
    stream_out['omega'] = omega
    
    stream_out['M_0_sim'] = M_0_sim
    stream_out['M_1_sim'] = M_1_sim
    stream_out['M_2_sim'] = M_2_sim    
    
    stream_out['C_0_sim'] = C_0_sim
    stream_out['C_1_sim'] = C_1_sim
    stream_out['C_2_sim'] = C_2_sim
    
    stream_out['mean_t'] = mean_t
    stream_out['c_t'] = c_t
    stream_out['c_check_f']  = c_f
    
    np.savez(foldername+filename, **stream_out)


def read(foldername, filename):
    """Reads the data evaluated from the simulation into a npz-file."""
    files = os.listdir(foldername)
    print filename
    
    for f in files:    
        if(f.find(filename)!=-1):
            print "File found!"
            print foldername + filename
            stream_in = np.load(foldername + filename)
     
            h0 = stream_in['h0'] 
            omega = stream_in['omega'] 

            M_0_sim = stream_in['M_0_sim']
            M_1_sim = stream_in['M_1_sim']
            M_2_sim = stream_in['M_2_sim'] 
            
            C_0_sim = stream_in['C_0_sim']
            C_1_sim = stream_in['C_1_sim_check']
            C_2_sim = stream_in['C_2_sim']
                
            mean_t = stream_in['mean_t']
            c_t = stream_in['c_t']
            c_f = stream_in['c_check_f']
            file_found = 1
            return (h0, omega, M_0_sim, M_1_sim, M_2_sim, 
                    C_0_sim, C_1_sim, C_1_sim, C_1_sim, C_2_sim, 
                    mean_t, c_t, c_f, c_f, c_f, file_found)
    file_found = 0
    print "File not found."
    return (0., 0., 
            np.zeros(4, dtype=float), np.zeros(4, dtype=float), 
            np.zeros(4, dtype=float), 
            np.zeros(3, dtype=float), np.zeros(3, dtype=float), 
            np.zeros(3, dtype=float), np.zeros(3, dtype=float), 
            np.zeros(3, dtype=float), 
            np.zeros((4,1), dtype=float), np.zeros((3,1),dtype=float), 
            np.zeros((3,1),dtype=float), np.zeros((3,1),dtype=float), 
            np.zeros((3,1),dtype=float), file_found)
      
def glue_read(foldername, filenames):
    """Opens npz-files containing the statistical evaluations of the 
    simulalations for single frequencies omega and creates an array containing
    all of them."""
    N_total = len(filenames)
    
    h0_list = [] 
    omega_list = []

    M_0_sim_list = []    
    M_1_sim_list = []
    M_2_sim_list = []    
    
    C_0_sim_list = []
    C_1_sim_list = []
    C_2_sim_list = []
    
    m_t_list  = []
    c_t_list  = []
    c_f_list  = []
    file_found_list = []
    
    for ii in range(0,N_total):
        (h0, omega, M_0_sim, M_1_sim, M_2_sim, 
         C_0_sim, C_1_sim_check, C_1_sim_indie,  C_1_sim_chopped, C_2_sim, 
         m_t, c_t, c_check_f, c_indie_f, c_chopped_f, 
         file_found) = read(foldername, filenames[ii])
        print filenames[ii]
        print ii

        h0_list.append(h0)
        omega_list.append(omega)
        
        M_0_sim_list.append(M_0_sim)        
        M_1_sim_list.append(M_1_sim)
        M_2_sim_list.append(M_2_sim)
        
        C_0_sim_list.append(C_0_sim)
        C_1_sim_list.append(C_1_sim_check)
        C_2_sim_list.append(C_2_sim)
        
        m_t_list.append(m_t)
        c_t_list.append(c_t)
        c_f_list.append(c_check_f)
        
        file_found_list.append(file_found)

    h0_list    = np.array(h0_list).T
    omega_list = np.array(omega_list).T
    
    M_0_sim_list = np.array(M_0_sim_list).T     
    M_1_sim_list = np.array(M_1_sim_list).T     
    M_2_sim_list = np.array(M_2_sim_list).T

    C_0_sim_list = np.array(C_0_sim_list).T
    C_1_sim_list = np.array(C_1_sim_list).T
    C_2_sim_list = np.array(C_2_sim_list).T

    file_found_list = np.array(file_found_list)

    return (h0_list, omega_list, M_0_sim_list, M_1_sim_list, M_2_sim_list, 
            C_0_sim_list, C_1_sim_list, C_1_sim_list,  C_1_sim_list, C_2_sim_list, 
            m_t_list, c_t_list, c_f_list, c_f_list, c_f_list, file_found_list)

def average_over_seeds(foldername, freq_max, suffix_list):
    """Loads the arrays containing time series, Fourier trafos and Fourier
    coefficients of mean activities and correlations for different seeds
    of the simulations and averages over them."""
    delta_freq = freq_max/10.
    
    def load_npz(suffix_general):
        datafile_names = []    
        for k in range(0,21):     #omega is the radian frequency, measured in Hz
            if (k==0): 
                omega = 0. 
                T_total = 30000.
            else:
                T_ext_approx = 1/(k * delta_freq)
                T_ext = int(T_ext_approx/params.dt_samp) * params.dt_samp
        
                omega = 1000. * 2* np.pi/T_ext  # 1000.* because dt_samp is measured in ms, but omega must be given in Hz, not in kHz!
                T_total = params.T
            print 'omega=', omega
            
            filename = ("plot_data_h0="+str(params.h0_default)+"_omega="+str(omega) 
                       +'_T=' + str(T_total) + suffix_general + ".npz")
                
            datafile_names.append(filename)    

        (h0, omegas, M0, M1, M2, C0, 
         C1, C1_indie, C1_chopped, C2, 
         m_t, c_t, C_f_full, C_indie_f_full, C_chopped_f_full, 
         files_found) = glue_read(foldername,datafile_names)
         
        return (h0, omegas, M0, M1, M2, C0, C1, C1_indie, C1_chopped, C2, 
                m_t, c_t, C_f_full, C_indie_f_full, C_chopped_f_full, files_found)
    
    #Separate the first suffix to create an array you can add on    
    (h0, omegas, M0_av, M1_av, M2_av, 
     C0_av, C1_av, C1_indie_av, C1_chopped_av, C2_av, 
     m_t_av, c_t_av, C_f_full_av, C_indie_f_full_av, C_chopped_f_full_av, 
     files_found_sum) = load_npz(suffix_list[0])
     
    for suffix_general in suffix_list[1:]:
        (h0, omegas, M0, M1, M2, C0, 
         C1, C1_indie, C1_chopped, C2, 
         m_t, c_t, C_f_full, C_indie_f_full, C_chopped_f_full, 
         files_found) = load_npz(suffix_general)
        
        # Add results only for those omega, for which the simulation has been successful (i.e. files_found=1)
        M0_av[:,np.nonzero(files_found)] += M0[:,np.nonzero(files_found)]        
        M1_av[:,np.nonzero(files_found)] += M1[:,np.nonzero(files_found)]
        M2_av[:,np.nonzero(files_found)] += M2[:,np.nonzero(files_found)]
        
        C0_av[:,np.nonzero(files_found)] += C0[:,np.nonzero(files_found)]
        C1_av[:,np.nonzero(files_found)] += C1[:,np.nonzero(files_found)]
        C2_av[:,np.nonzero(files_found)] += C2[:,np.nonzero(files_found)]
        
        print 'np.nonzero(files_found)[0]=', np.nonzero(files_found)[0]        
        
        for ii in np.nonzero(files_found)[0]:
            #If simulations for a certain omega are missing so far, we have to treat this case separately
            if(ii in np.nonzero(files_found_sum)[0]):
                m_t_av[ii] += m_t[ii]
                c_t_av[ii] += c_t[ii]
                C_f_full_av[ii] += C_f_full[ii]
            else:
                m_t_av[ii] = m_t[ii]
                c_t_av[ii] = c_t[ii]
                C_f_full_av[ii]   = C_f_full[ii] 
                
        print 'C2=', C2
        print 'C2_av=', C2_av
        
        print 'files_found=', files_found
        print 'files_found_sum', files_found_sum

        files_found_sum += files_found
    
    print 'len(suffix_list)=', len(suffix_list)    
    print "files_found_sum=", files_found_sum
        
    print 'np.nonzero(files_found_sum)=', np.nonzero(files_found_sum)
    print 'np.nonzero(files_found_sum)[0]=', np.nonzero(files_found_sum)[0]    
    
    h0 = h0[np.nonzero(files_found_sum)[0]] # Keep only those values, for which there are actually successfull simulations
    omegas = omegas[np.nonzero(files_found_sum)[0]]       
    #Normalize all entries to the number of successfull simulations
    M0_av = M0_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    print 'np.nonzero(files_found_sum)=', np.nonzero(files_found_sum)
    M1_av = M1_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    M2_av = M2_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    
    C0_av = C0_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    C1_av = C1_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    C2_av = C2_av[:,np.nonzero(files_found_sum)[0]]/files_found_sum[np.nonzero(files_found_sum)[0]]
    
    for ii in np.nonzero(files_found_sum)[0]:
        m_t_av[ii] = m_t_av[ii]/files_found_sum[ii]
        c_t_av[ii] = c_t_av[ii]/files_found_sum[ii]    
        C_f_full_av[ii]   = C_f_full_av[ii]/files_found_sum[ii]
        
    return (h0, omegas, M0_av, M1_av, M2_av, C0_av, C1_av, C1_av, C1_av, C2_av, 
            m_t_av, c_t_av, C_f_full_av, C_f_full_av, C_f_full_av)

def calc_theory_num(N_omegas, freq_max):
    """Calculates the mean activities and pairwise corellations for one period 
    after the transient phase numerically and determines the Fourier
    coefficients up to order 2 for N_omega different perturbing frequencies 
    equally spaced from 0 to freq_max."""    
    delta_freq  = freq_max/N_omegas
    
    omegas  = np.zeros(N_omegas, dtype=float)
    M0_list = np.zeros((2,N_omegas), dtype=complex)
    M1_list = np.zeros((2,N_omegas), dtype=complex)
    M2_list = np.zeros((2,N_omegas), dtype=complex)
    
    C0_list = np.zeros((5,N_omegas), dtype=complex)
    C1_list = np.zeros((5,N_omegas), dtype=complex)
    C2_list = np.zeros((5,N_omegas), dtype=complex)
    
    t_time_list = []    
    m_time_list = []
    c_time_list = []
    
    for ii in range(0,N_omegas):
        t0 = 0
        N_transient = int(10 * params.tau_m/params.dt_samp) 
        if(ii==0):
            omega        = 0.
            omega_kHz    = 0.
            N_per_period = 100
            N_eff        = N_per_period
        else:
            freq_approx_kHz = ii * delta_freq/1000.
            N_per_period    = int(1./(freq_approx_kHz * params.dt_samp))
            N_eff           = N_per_period
            N_transient     = (int(N_transient/N_per_period)+1.) * N_per_period
                
            omega_kHz = 2. * np.pi/(N_per_period * params.dt_samp)            
            omega     = 2. * np.pi/(N_per_period * params.dt_samp) * 1000.
    
        t1 = (N_transient + N_per_period) * params.dt_samp
        m_linear  = 0 #If 1, in the ODE for the mean activity, nonlinear terms are omitted, if 0, full ODEs are considered
        c_linear  = 0 #If 1, in the ODE for the correlation, nonlinear terms are omitted, if 0, full ODEs are considered
        quadratic = 0 #If 1 and c_linear = 0, in addition to the linear order, the quadratic order is considered in the ODE 
                      #for the correlations. If c_linear = 1, value is irrelevant
        (t_list, m_num_list, c_num_list, 
         m0, c0) = integrate_m_c_ode(t0,t1,params.dt_samp, 
                                     m_linear, c_linear, quadratic, 
                                     h0 = params.h0_default, omega = omega)
        
        t_time_list.append(np.array(t_list[N_transient:N_transient + N_eff])-t_list[N_transient])
        m_time_list.append(m_num_list[:,N_transient:N_transient + N_eff])
        c_time_list.append(c_num_list[:,N_transient:N_transient + N_eff])
        
        for jj in range(0,5):        
            C0_list[jj,ii] = np.mean(c_num_list[jj,N_transient:N_transient + N_eff])
            C1_list[jj,ii] = np.mean((c_num_list[jj,N_transient:N_transient + N_eff] - c0[jj]) 
                                      * np.exp(-1j * omega_kHz *t_list[N_transient:N_transient + N_eff]))
            C2_list[jj,ii] = np.mean((c_num_list[jj,N_transient:N_transient + N_eff] - c0[jj]) 
                                      * np.exp(-2j * omega_kHz *t_list[N_transient:N_transient + N_eff]))
        
        for jj in range(0,2):
            M0_list[jj,ii] = np.mean(m_num_list[jj,N_transient:N_transient + N_eff])
            M1_list[jj,ii] = np.mean((m_num_list[jj,N_transient:N_transient + N_eff] - m0[jj]) 
                                      * np.exp(-1j * omega_kHz *t_list[N_transient:N_transient + N_eff]))
            M2_list[jj,ii] = np.mean((m_num_list[jj,N_transient:N_transient + N_eff] - m0[jj]) 
                                      * np.exp(-2j * omega_kHz *t_list[N_transient:N_transient + N_eff]))
        
        omegas[ii] = omega  
    
    return omegas, M0_list, M1_list, M2_list, C0_list, C1_list, C2_list, t_time_list, m_time_list, c_time_list


def calc_theory(N_omega, freq_max):
    """Calculates the first two Fourier coefficients of the first two cumulants 
    in linear perturbation order of the mean field theory for N_omega different 
    perturbing frequencies equally spaced from 0 to freq_max."""
    omega_ext_max = 1000. * 2* np.pi * 2.* freq_max  # 1000.* because freq_max is given in kHz, but omega is always in Hz.   
    
    M0_list = np.zeros((2,N_omega), dtype=complex)    
    M1_list = np.zeros((2,N_omega), dtype=complex)
    C0_list = np.zeros((5,N_omega), dtype=complex)
    C1_list = np.zeros((5,N_omega), dtype=complex)
    
    C1_eigen_list  = np.zeros((5,N_omega), dtype=complex)

    C1_S_m_list = np.zeros((5,N_omega), dtype=complex)    
    C1_S_h_list = np.zeros((5,N_omega), dtype=complex)    
    C1_a_m_list = np.zeros((5,N_omega), dtype=complex)    
        
    omegas_theo  = np.linspace(0.,omega_ext_max, N_omega, endpoint = False)
    
    for l in range(0,N_omega):
        omega = omegas_theo[l]    
        g = ginzburg(np.array([params.N_E,params.N_I,params.N_X]), 
                     params.J_array, params.K_array, params.m_x, params.theta,
                     np.array([params.sigma_noise_E,params.sigma_noise_I]),
                     omega, params.h0_default)
        
        ms, cs = g.solve_self_consistently_corr(params.m_exp)
        mu, sigma2 = g.mu_sigma_from_m_corr(ms, cs)
        
        dev_mean_freq, ente, gans = g.M1_linear_perturb_theory(ms, params.tau_m)
        M1_list[:,l] = dev_mean_freq[:2]
        
        ##########################################
        ### For the correlations: ################
        ##########################################
        (C1, C1_eigen, schwan1, schwan2, schwan3, schwan4, schwan5, 
         C1_S_m, C1_S_h, C1_a_m) = g.C1_linear_perturb_theory(ms, params.tau_m)
        #Fill the list containing the analytically calculated values for c
        C1_list[EE,l] = C1[E,E]
        C1_list[EI,l] = C1[E,I]
        C1_list[II,l] = C1[I,I]
        C1_list[EX,l] = C1[E,X]
        C1_list[IX,l] = C1[I,X]
        
        C1_S_m_list[EE,l] = C1_S_m[E,E]
        C1_S_m_list[EI,l] = C1_S_m[E,I]
        C1_S_m_list[II,l] = C1_S_m[I,I]
        C1_S_m_list[EX,l] = C1_S_m[E,X]
        C1_S_m_list[IX,l] = C1_S_m[I,X]
       
        C1_S_h_list[EE,l] = C1_S_h[E,E]
        C1_S_h_list[EI,l] = C1_S_h[E,I]
        C1_S_h_list[II,l] = C1_S_h[I,I]
        C1_S_h_list[EX,l] = C1_S_h[E,X]
        C1_S_h_list[IX,l] = C1_S_h[I,X]         
        
        C1_a_m_list[EE,l] = C1_a_m[E,E]
        C1_a_m_list[EI,l] = C1_a_m[E,I]
        C1_a_m_list[II,l] = C1_a_m[I,I]
        C1_a_m_list[EX,l] = C1_a_m[E,X]
        C1_a_m_list[IX,l] = C1_a_m[I,X]        
        
        C1_eigen_list[EE,l] = C1_eigen[E,E]
        C1_eigen_list[EI,l] = C1_eigen[E,I]
        C1_eigen_list[II,l] = C1_eigen[I,I]
        C1_eigen_list[EX,l] = C1_eigen[E,X]
        C1_eigen_list[IX,l] = C1_eigen[I,X]        
        
        for ii in range(0,5):        
            C0_list[ii,l] = cs[ii]
        for ii in range(0,2):        
            M0_list[ii,l] = ms[ii]
        
    return omegas_theo, M0_list, M1_list, C0_list, C1_list, C1_eigen_list, C1_S_m_list, C1_S_h_list, C1_a_m_list

def load_data_calc_theory_comp_num(datafolder, freq_max, suffix_list):
    (h0_new, omegas_new, 
     M_0_sim_new, M_1_sim_new, M_2_sim_new, 
     C_0_sim_new, C_1_sim_check_new, C_1_sim_indie_new, C_1_sim_chopped_new, C_2_sim_new, 
     m_sim_t_new, c_sim_t_new, 
     C_sim_check_f_full_new, C_sim_indie_f_full_new, C_sim_chopped_f_full_new
    ) = average_over_seeds(datafolder, freq_max, suffix_list)
    (omegas_theo, m_const_theo, M1_theo, 
     c_const_theo, c_delsig_list, schwan, 
     C_S_m_list, C_S_h_list, C_a_m_list) = calc_theory(1000, freq_max)
    (omegas_num, 
     M0_list_num, M1_list_num, M2_list_num, 
     C0_list_num, C1_list_num, C2_list_num, 
     times_num, m_list_num, c_list_num) = calc_theory_num(25, 2000 * freq_max)
    
    g = ginzburg(np.array([params.N_E,params.N_I,params.N_X]), 
                 params.J_array, params.K_array, params.m_x, params.theta,
                 np.array([params.sigma_noise_E,params.sigma_noise_I]),
                 0., params.h0_default)
    eff_connect, EW_connect, EV_connect = g.effective_connectivity(params.m_exp)
    
    f_res_mean = np.abs(np.imag(EW_connect[0]))*1000./(2*np.pi*params.tau_m)
    f_res_corr = 2.*f_res_mean
    
    return (h0_new, omegas_new, 
            M_0_sim_new, M_1_sim_new, M_2_sim_new, 
            C_0_sim_new, C_1_sim_check_new, C_1_sim_indie_new, C_1_sim_chopped_new, C_2_sim_new, 
            m_sim_t_new, c_sim_t_new, 
            C_sim_check_f_full_new, C_sim_indie_f_full_new, C_sim_chopped_f_full_new,
            omegas_theo, m_const_theo, M1_theo, 
            c_const_theo, c_delsig_list, schwan, 
            C_S_m_list, C_S_h_list, C_a_m_list,
            omegas_num, 
            M0_list_num, M1_list_num, M2_list_num, 
            C0_list_num, C1_list_num, C2_list_num, 
            times_num, m_list_num, c_list_num,
            f_res_mean, f_res_corr)          

#datafolder = '../data_LFP_new/'
#suffix_general = '_mult_theta_2016_1_4_11_13_42'
#suffix_general = '_mult_theta_symmetric_test'
#suffix_general = '_mult_theta_2015_10_2_8_41_55'
#suffix_general = '_mult_theta_2015_12_17_12_16_49'
#suffix_general = '_mult_theta_2015_11_13_12_37_22'
#suffix_general = '_mult_theta_2016_2_3_13_22_58'
#suffix_general = '_mult_theta_2016_2_5_14_49_41'
#suffix_general = '_mult_theta_2016_2_9_11_31_6'
#suffix_general = '_mult_theta_2016_2_9_23_7_34'
#suffix_general = '_mult_theta_2016_2_15_16_15_53'
#suffix_general = '_mult_theta_2016_2_26_11_17_40'
#suffix_general = '_mult_theta_2016_2_28_20_58_22'
#suffix_general = '_mult_theta_2016_2_29_13_42_3'
#suffix_general = '_mult_theta_2016_3_2_8_47_48'
#suffix_general = '_mult_theta_seed=25_2016_3_5_16_32_56'
#suffix_general = '_mult_theta_seed=13_2016_3_13_12_53_40'
#suffix_general = '_ente'
#suffix_general = '_moritz'
#suffix_general = '_linear_test'
#suffix_general = '_2016_3_17_12_45_20_seeds=1_13_25'
#suffix_general = '_mult_theta_2016_6_10_9_20_16'
#suffix_general = '_mult_theta_2016_7_1_12_53_39'
#suffix_general = '_mult_theta_2016_7_6_12_10_40'

#freq_max   = 0.200            # Maximal frequency in kHz

#suffix_list = np.array(['_mult_theta_seed=1_2016_3_17_12_45_20', '_mult_theta_seed=13_2016_3_13_12_53_40', '_mult_theta_seed=25_2016_3_5_16_32_56'])
#suffix_list = np.array(['_mult_theta_seed=25_2016_3_5_16_32_56', '_mult_theta_seed=13_2016_3_13_12_53_40'])
#suffix_list = np.array(['_mult_theta_seed=1_2016_6_10_9_20_16','_mult_theta_seed=13_2016_6_10_9_20_16','_mult_theta_seed=25_2016_6_10_9_20_16','_mult_theta_seed=37_2016_6_10_9_20_16','_mult_theta_seed=49_2016_6_10_9_20_16'])
#suffix_list = np.array(['_mult_theta_seed=1_2016_6_10_9_20_16','_mult_theta_seed=13_2016_6_10_9_20_16','_mult_theta_seed=25_2016_6_10_9_20_16','_mult_theta_seed=37_2016_6_10_9_20_16','_mult_theta_seed=49_2016_6_10_9_20_16', '_mult_theta_seed=61_2016_6_23_23_24_0', '_mult_theta_seed=73_2016_6_23_23_24_0', '_mult_theta_seed=85_2016_6_23_23_24_0', '_mult_theta_seed=97_2016_6_23_23_24_0', '_mult_theta_seed=109_2016_6_23_23_24_0'])
#suffix_list = np.array(['_mult_theta_seed=1_2016_7_1_12_53_39'])
#suffix_list = np.array(['_mult_theta_seed=1_2016_7_6_12_10_40', '_mult_theta_seed=13_2016_7_6_12_10_40', '_mult_theta_seed=25_2016_7_6_12_10_40'])

#(h0_new, omegas_new, 
# M_0_sim_new, M_1_sim_new, M_2_sim_new, 
# C_0_sim_new, C_1_sim_check_new, C_1_sim_indie_new, C_1_sim_chopped_new, C_2_sim_new, 
# m_sim_t_new, c_sim_t_new, 
# C_sim_check_f_full_new, C_sim_indie_f_full_new, C_sim_chopped_f_full_new,
# omegas_theo, m_const_theo, M1_theo, 
# c_const_theo, c_delsig_list, schwan, 
# C_S_m_list, C_S_h_list, C_a_m_list,
# omegas_num, 
# M0_list_num, M1_list_num, M2_list_num, 
# C0_list_num, C1_list_num, C2_list_num, 
# times_num, m_list_num, c_list_num,
# f_res_mean, f_res_corr) = load_data_calc_theory_comp_num(datafolder, freq_max, 
#                                                          suffix_list)

#single_pop = 0
#plot_collection = plot_freq_dependence_mean_corr(omegas_new, 
#                               M_0_sim_new, M_1_sim_new, M_2_sim_new, 
#                               C_0_sim_new, 
#                               C_1_sim_check_new, 
#                               C_2_sim_new, 
#                               f_res_mean, f_res_corr, omegas_theo, 
#                               m_const_theo, M1_theo,
#                               c_const_theo, 
#                               c_delsig_list, C_S_m_list, C_S_h_list, C_a_m_list,
#                               omegas_num, 
#                               M0_list_num, M1_list_num, M2_list_num, 
#                               C0_list_num, C1_list_num, C2_list_num,
#                               single_pop,
#                               suffix_general)

#plot_collection.plot_first_Fourier_comp_corr_mean(E, 'exc.',EE, 'ee', 1)
#plot_collection.plot_first_Fourier_comp_corr_mean(I, 'inh.',II, 'ii', 2)
#
#plot_collection.plot_three_Fourier_mean(3)
#
#plot_collection.plot_three_Fourier_corr_comp(EE, 'ee', 4)    
#plot_collection.plot_three_Fourier_corr_comp(II, 'ii', 5)    
#plot_collection.plot_three_Fourier_corr_comp(EI, 'ei', 6)