# -*- coding: utf-8 -*-
"""
Created on Fri Oct 16 13:44:17 2015

@author: kuehn
"""
import matplotlib as mp
import matplotlib.pylab as plb
import os

from plotfuncs import *

import numpy as np
from scipy.stats import poisson
from scipy.stats import gamma
from scipy.special import gamma as gamma_fct

import scipy.special as sps

from scipy.optimize import fsolve 

from ginzburg_theory_osc_input import ginzburg
import parameters_ginzburg_asymm_LFP_new_noisy_mult_theta as params

scale = 1.
width = 6.83 # inches for single column in PLoS


def prob_UE_corr_network_for_uncorr_network_expected(m,c,T,n0):

    prob = 1 - poisson.cdf(n0,(m**2+c)*T)
#    eps = 10. **(-12)    
#    prob = poisson.pmf(n0,(m**2+c)*T)
#        
#    n = n0 + 1    
#    while(poisson.pmf(n,(m**2+c)*T)/prob > eps):
#        #print 'n=', n
#        #print 'prob=', prob
#        #print 'add_prob=', poisson.pmf(n,(m**2+c)*T)        
#        prob = prob + poisson.pmf(n,(m**2+c)*T)
#        n = n+1
    
    #print 'n_final=', n-1    
    return prob
    
def n0_finder(m,T,p0):
#    mu = m**2 * T
#    n0 = int(mu)
#    p = prob_UE_corr_network_for_uncorr_network_expected(m,0.,T,n0)
#    while(p>p0):
#        n0 = n0 + 1
#        p = prob_UE_corr_network_for_uncorr_network_expected(m,0.,T,n0)
#    return n0
    return poisson.isf(p0,m**2*T)

def prob_UE_continuous(m,c,T,n0):
    lbda  = (m**2+c)*T  
    return sps.gammainc(n0+1,lbda)

def poisson_continuous_inv(m,T,p):
    lbda = m**2*T
    def iter_func(x):
        return sps.gammainc(x+1,lbda) - p
        
    x = poisson.isf(p,lbda)  #inverse function of the cumulative poisson distr. for the starting value
    x = fsolve(iter_func,  x)
    return x

def del_lbda_prob_UE_continuous(m,T,p):
    result = np.zeros_like(m)
    for ii in range(0,len(m)):    
        lbda  = m[ii]**2*T
        s = poisson_continuous_inv(m[ii],T,p)
        result[ii] = lbda**(s) * np.exp(-lbda)/gamma_fct(s+1)
    return result

def UE_prob(m_list, c_list, T, p0):
    n0_list      = np.zeros_like(m_list)
    UE_prob_list = np.zeros_like(m_list)
    for ii in range(0,len(n0_list)):
        n0               = poisson_continuous_inv(m_list[ii],T,p0)        
        n0_list[ii]      = n0
        UE_prob_list[ii] = prob_UE_continuous(m_list[ii],c_list[ii],T,n0)
        
    return n0_list, UE_prob_list 

def UE_prob_biased(m_list, c_list, T, p0):
    m_mean       = np.mean(m_list)    
    n0           = poisson_continuous_inv(m_mean,T,p0)    
    
    n0_list      = n0 * np.ones_like(m_list)
    UE_prob_list = np.zeros_like(m_list)

    for ii in range(0,len(n0_list)):
        UE_prob_list[ii] = prob_UE_continuous(m_list[ii],c_list[ii],T,n0)
        
    return n0_list, UE_prob_list 

def plot_UE(m_list, c_list, h0, omega, T, p0, figure_start_number, suffix_general):
    t_list = np.linspace(0,2.*np.pi * 1000./omega, len(m_list), endpoint=False)
    
    n0_list, UE_prob_list = UE_prob(m_list,c_list,T,p0)
    
    #m_mean = np.mean(m_list)
    n0_biased_list, UE_prob_biased_list = UE_prob_biased(m_list,c_list,T,p0)
    #n0_biased_list, UE_prob_biased_list = UE_prob_biased(m_list,np.zeros_like(c_list),T,p0)    
    
    plb.figure(figure_start_number + 1)
    plb.plot(t_list, n0_list)
    plb.title('n0-plot')
     
    plb.figure(figure_start_number + 2)
    plb.plot(t_list, UE_prob_list)
    plb.xlabel('$t [ms]$')
    plb.ylabel('prob for UE')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    figurename = '/UE_analysis/Prob_for_UE_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')
    
    plb.figure(figure_start_number + 3)
    plb.plot(t_list, UE_prob_biased_list)
    plb.xlabel('$t [ms]$')
    plb.ylabel('prob for UE for mean act. assumend to be const.')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    figurename = '/UE_analysis/Prob_for_UE_biased_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')    
    
#    plb.figure(figure_start_number + 5)
#    plot(t_list, np.real(c_sim_t_new[1][EE]))
#    plb.plot(t_list, np.real(c_list))
#    plb.xlabel('$t [ms]$')
#    plb.ylabel('$c_{ee}$')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')

    Tafel_Fabrik = create_fig_plos(figure_start_number + 5, scale, width, 1, 1, hoffset=0.15) 
    
    
    ################################################################################################
    ginz = ginzburg(np.array([params.N_E,params.N_I,params.N_X]), 
                    params.J_array, params.K_array, params.m_x, params.theta, 
                    np.array([params.sigma_noise_E,params.sigma_noise_I]) ,
                    omega, params.h0_default)                    
    ms, cs = ginz.solve_self_consistently_corr(params.m_exp)
    (C_delsig, schwan, schwan1, schwan2, schwan30, schwan31, schwan32, C_S_m, C_S_h, 
     C_a_m) = ginz.C1_linear_perturb_theory(ms, params.tau_m)
    dev_mean_freq, ente, gans = ginz.M1_linear_perturb_theory(ms, params.tau_m)    
    C_ee_theo = C_delsig[0,0]     
    ################################################################################################

    Tafel_Fabrik.new_panel(0, 0, r'A')
    #rcParams['axes.labelsize'] = scale*50
    #rcParams['axes.labelsize'] = scale*15
    #rcParams['axes.titlesize'] = scale*10
    rcParams['legend.fontsize'] = scale*20
    xticks(fontsize = scale*15)
    yticks(fontsize = scale*15)
    #plb.plot(t_list, np.real(c_list)*10**5, label='$c_{ee}(t)$')
    #plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
    plb.plot(t_list, (np.real(c_list)-np.mean(np.real(c_list)))*10**5, label='$c_{ee}(t)$')
    plb.plot(t_list, np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.))*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')    
    plb.xlabel('$t [ms]$', fontsize = 20)
    plb.ylabel('$\delta c_{ee}[10^{-5}]$', fontsize = 20)
    plb.title('$\omega=$' + str(round(omega/(2.*np.pi))) + '$\cdot 2\pi$', fontsize = 20)    

    legend(loc = 0)
    
    figurename = '/UE_analysis/c_time_course_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')
    
    plb.figure(figure_start_number + 6)
    #plot(t_list, np.real(m_sim_t_new[1][E]))
    plb.plot(t_list, m_list)
    plb.xlabel('$t [ms]$')
    plb.ylabel('$m_{e}$')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    figurename = '/UE_analysis/m_time_course_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')
    
    UE_prob_FT = np.fft.fft(UE_prob_list)
    freq       = np.fft.fftfreq(len(UE_prob_list))
    
    plb.figure(figure_start_number + 7)
    plb.plot(freq, np.abs(UE_prob_FT),'.')
    
    figurename = '/UE_analysis/Prob_for_UE_FT_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')

    UE_prob_biased_FT = np.fft.fft(UE_prob_biased_list)
    freq_biased       = np.fft.fftfreq(len(UE_prob_biased_list))
    
    plb.figure(figure_start_number + 8)
    plb.plot(freq_biased, np.abs(UE_prob_biased_FT),'.')
    
    figurename = '/UE_analysis/Prob_for_UE_biased_FT_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')
    
    # Compound pictures:
    
    n_horz_panels = 2
    n_vert_panels = 2
    
    Tafel_Fabrik = create_fig_plos(figure_start_number + 9, scale, width, n_horz_panels, n_vert_panels)    

    rcParams['axes.labelsize']  = scale*10
    
    Tafel_Fabrik.new_panel(0, 0, r'A')
    plb.plot(t_list, c_list*10**5)
    #plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
    plb.xlabel('$t [ms]$')
    plb.ylabel('$c_{ee}[10^{-5}]$')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')    
    
    Tafel_Fabrik.new_panel(1, 0, r'B')
    plb.plot(t_list, m_list*10**2)
    #plb.plot(t_list, (np.imag(dev_mean_freq[0] * np.exp(1j * omega * t_list/1000.)) + ms[0])*10**2, label='$m_{e, \mathrm{theo.}}(t)$')
    plb.xlabel('$t [ms]$')
    plb.ylabel('$m_{e}[10^{-2}]$')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
        
    rcParams['legend.fontsize'] = scale*6
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(0, 1, r'C')
    plb.plot(t_list, UE_prob_list)
    plb.xlabel('$t [ms]$')
    plb.ylabel('P(UE)')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    rcParams['legend.fontsize'] = scale*6
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(1, 1, r'D')
    plb.plot(t_list, UE_prob_biased_list)
    plb.xlabel('$t [ms]$')
    plb.ylabel('P(UE) assuming const. m')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    rcParams['legend.fontsize'] = scale*6
    legend(loc=0)
    
    figurename = '/UE_analysis/m_c_biased_unbiased/m_and_c_and_prob_for_UE_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general
        
    savefig(params.figurepath_default + figurename + '.pdf')
    savefig(params.figurepath_default + figurename + '.eps')
    
#    n_horz_panels = 2
#    n_vert_panels = 2
#    
#    Tafel_Fabrik = create_fig_plos(figure_start_number + 9, scale, width, n_horz_panels, n_vert_panels)    
#
#    rcParams['axes.labelsize']  = scale*10
#    
#    Tafel_Fabrik.new_panel(0, 0, r'A')
#    plb.plot(t_list, np.real(c_list)*10**5)
#    plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
#    plb.xlabel('$t [ms]$')
#    plb.ylabel('$c_{ee}[10^{-5}]$')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')    
#    
#    Tafel_Fabrik.new_panel(1, 0, r'B')
#    plb.plot(t_list, m_list*10**2)
#    plb.plot(t_list, (np.imag(dev_mean_freq[0] * np.exp(1j * omega * t_list/1000.)) + ms[0])*10**2, label='$m_{e, \mathrm{theo.}}(t)$')
#    plb.xlabel('$t [ms]$')
#    plb.ylabel('$m_{e}[10^{-2}]$')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
#        
#    rcParams['legend.fontsize'] = scale*6
#    legend(loc=0)
#    
#    Tafel_Fabrik.new_panel(0, 1, r'C')
#    plb.plot(t_list, UE_prob_list)
#    plb.xlabel('$t [ms]$')
#    plb.ylabel('P(UE)')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
#    
#    rcParams['legend.fontsize'] = scale*6
#    legend(loc=0)
#    
#    Tafel_Fabrik.new_panel(1, 1, r'D')
#    plb.plot(t_list, UE_prob_biased_list)
#    plb.xlabel('$t [ms]$')
#    plb.ylabel('P(UE) assuming const. m')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
#    
#    rcParams['legend.fontsize'] = scale*6
#    legend(loc=0)
    
def plot_UE_comp_sim_theo(m_sim_list, c_sim_list, m_theo_list, c_theo_list, h0, omega, T, p0, figure_start_number, suffix_general):
#    t_sim_list = np.linspace(0,2.*np.pi * 1000./omega, len(m_sim_list), endpoint=False)
#    t_theo_list = np.linspace(0,2.*np.pi * 1000./omega, len(m_theo_list), endpoint=False)
    t_sim_list = np.linspace(0,1., len(m_sim_list), endpoint=False)
    t_theo_list = np.linspace(0,1., len(m_theo_list), endpoint=False)
    
    n0_sim_list, UE_prob_sim_list = UE_prob(m_sim_list,c_sim_list,T,p0)
    UE_prob_sim_lin_list = p0 + del_lbda_prob_UE_continuous(m_sim_list,T,p0) * T * c_sim_list
    n0_biased_sim_list, UE_prob_biased_sim_list = UE_prob_biased(m_sim_list,c_sim_list,T,p0)
    (n0_corr_const_sim_list, 
     UE_prob_corr_const_sim_list) = UE_prob(m_sim_list, 
                                            np.mean(c_sim_list) * np.ones_like(
                                            c_sim_list),T,p0)    
    
    (n0_mean_const_sim_list, 
     UE_prob_mean_const_sim_list) = UE_prob(np.mean(m_sim_list) * np.ones_like(
                                             m_sim_list), c_sim_list,T,p0)    
    
    n0_theo_list, UE_prob_theo_list = UE_prob(m_theo_list,c_theo_list,T,p0)
    UE_prob_theo_lin_list = p0 + del_lbda_prob_UE_continuous(m_theo_list,T,p0) * T * c_theo_list
    n0_biased_theo_list, UE_prob_biased_theo_list = UE_prob_biased(m_theo_list,c_theo_list,T,p0)
    (n0_corr_const_theo_list, 
     UE_prob_corr_const_theo_list) = UE_prob(m_theo_list, 
                                            np.mean(c_theo_list) * np.ones_like(
                                            c_theo_list),T,p0)
    
    (n0_mean_const_theo_list, 
     UE_prob_mean_const_theo_list) = UE_prob(np.mean(m_theo_list) * np.ones_like(
                                             m_theo_list), c_theo_list,T,p0)
                                             
    m0_plot_F = np.mean(m_theo_list)
                                             
    #lbda_list = np.array([0.0001 * T, 0.001 * T, lbda_m-1., lbda_m, lbda_m + 1.,0.003 * T])
    m_list_plot_F = np.array([0.01, 0.03, m0_plot_F-1./T, m0_plot_F, m0_plot_F + 1./T,0.05])    
    lbda_list = m_list_plot_F**2 * T
    
    s_list_1 = poisson_continuous_inv(m_list_plot_F,T,0.01)
    s_list_5 = poisson_continuous_inv(m_list_plot_F,T,0.05)
    s_list_50 = poisson_continuous_inv(m_list_plot_F,T,0.5)
    s_list_95 = poisson_continuous_inv(m_list_plot_F,T,0.95)
#    s_list_99 = poisson_continuous_inv(m_list_plot_F,T,0.99)

    
    n_horz_panels = 2
    n_vert_panels = 3
    
    Tafel_Fabrik = create_fig_plos(figure_start_number + 9, scale, width, n_horz_panels, n_vert_panels)    
    
    
    rcParams['legend.fontsize'] = scale*8
    rcParams['axes.labelsize']  = scale*10
        
    
    Tafel_Fabrik.new_panel(0, 0, r'A')
    plb.plot(t_sim_list, c_sim_list*10**5, label='sim.', color=black)
    plb.plot(t_theo_list, c_theo_list*10**5, label='lin. theo.', color=midgray)
    #plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$c_{ee}[10^{-5}]$')
    axis_min_y = min(np.min(c_sim_list*10**5), np.min(c_theo_list*10**5))
    axis_max_y = min(np.max(c_sim_list*10**5), np.max(c_theo_list*10**5))
    yticks(np.arange(int(axis_min_y*10.)/10.,int(axis_max_y*10.)/10.+0.2, 0.2))
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    legend(loc=0)    
    
    Tafel_Fabrik.new_panel(1, 0, r'B')
    plb.plot(t_sim_list, m_sim_list*10**2, label='sim', color=black)
    plb.plot(t_theo_list, m_theo_list*10**2, label='lin. theo.', color=midgray)
    #plb.plot(t_list, (np.imag(dev_mean_freq[0] * np.exp(1j * omega * t_list/1000.)) + ms[0])*10**2, label='$m_{e, \mathrm{theo.}}(t)$')
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$m_{e}[10^{-2}]$')
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
        
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(0, 1, r'C')
    plb.plot(t_sim_list, UE_prob_sim_list, label='sim.', color=black)
    plb.plot(t_theo_list, UE_prob_theo_list, label='lin. theo.', color=midgray)
    
    plb.plot(t_sim_list, UE_prob_sim_lin_list, '--', label='sim.,lin.', color=black)
    plb.plot(t_theo_list, UE_prob_theo_lin_list, '--', label='lin. theo., lin.', color=midgray)

    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\lambda}^{\mathrm{UE}}$')
    axis_min_y = min(np.min(UE_prob_sim_list), np.min(UE_prob_theo_list), np.min(UE_prob_sim_lin_list), np.min(UE_prob_theo_lin_list))
    axis_max_y = max(np.max(UE_prob_sim_list), np.max(UE_prob_theo_list), np.max(UE_prob_sim_lin_list), np.max(UE_prob_theo_lin_list))
    yticks(np.arange(int(axis_min_y*1000.)/1000.,int(axis_max_y*1000.)/1000.+0.001, 0.001))    
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(1, 1, r'D')
    plb.plot(t_sim_list, UE_prob_biased_sim_list, label='sim., const. mean', color=black)
    plb.plot(t_theo_list, UE_prob_biased_theo_list, label='lin. theo., const. mean', color=midgray)    
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\overline{\lambda}}^{\mathrm{UE}}$ ($\mathrm{constant} \ n_{0}$)')
    axis_min_y = min(np.min(UE_prob_biased_sim_list), np.min(UE_prob_biased_theo_list))
    axis_max_y = max(np.max(UE_prob_biased_sim_list), np.max(UE_prob_biased_theo_list))
    yticks(np.arange(int(axis_min_y*10.)/10.,int(axis_max_y*10.)/10.+0.1, 0.1))
    #yticks(np.array([0.,0.1,0.2,0.25]))
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    legend(loc=0)
    
    rcParams['legend.fontsize'] = scale*6
    
    Tafel_Fabrik.new_panel(0, 2, r'E')
    plb.plot(t_sim_list, UE_prob_corr_const_sim_list,'--', label='sim., const. cov.', color=black)
    plb.plot(t_theo_list, UE_prob_corr_const_theo_list,'--', label='lin. theo., const. cov.', color=midgray)
    
    plb.plot(t_sim_list, UE_prob_mean_const_sim_list, label='sim., const. mean', color=black)
    plb.plot(t_theo_list, UE_prob_mean_const_theo_list, label='lin. theo., const. mean', color=midgray)    
    
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\lambda}^{\mathrm{UE}}$')

    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    axis_min_y = min(np.min(UE_prob_corr_const_sim_list), np.min(UE_prob_corr_const_theo_list), np.min(UE_prob_mean_const_sim_list), np.min(UE_prob_mean_const_theo_list))
    axis_max_y = max(np.max(UE_prob_corr_const_sim_list), np.max(UE_prob_corr_const_theo_list), np.max(UE_prob_mean_const_sim_list), np.max(UE_prob_mean_const_theo_list))
    yticks(np.arange(int(axis_min_y*1000.)/1000.,int(axis_max_y*1000.)/1000.+0.001, 0.001))    
    
    legend(loc=0)
    
    rcParams['legend.fontsize'] = scale*7
    
    Tafel_Fabrik.new_panel(1, 2, r'F')
    plb.plot(lbda_list,s_list_1, label = r'$p_{0}=1\%$', color=black)
    plb.plot(lbda_list,s_list_5, label = r'$p_{0}=5\%$', color=darkgray)
    plb.plot(lbda_list,s_list_50, label = r'$p_{0}=50\%$', color=midgray)
    plb.plot(lbda_list,s_list_95, label = r'$p_{0}=95\%$', color=lightgray)
    #plb.plot(lbda_list,s_list_99, label = r'$p_{0}=99\%$')
    plb.plot(lbda_list, lbda_list, '--', linewidth=2, color = black)#
    plb.xlabel(r'$\lambda$')
    plb.ylabel(r'$n_{0}$')
#    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')

    plb.xticks([0.01**2*T, 0.03**2*T, m0_plot_F**2*T ,0.05**2*T],[0.01**2*T, 0.03**2*T, r'$\overline{\lambda}_{m}$' ,0.05**2*T])
    axis_min_y = min(np.min(s_list_1), np.min(s_list_5), np.min(s_list_50), np.min(s_list_95))
    axis_max_y = max(np.max(s_list_1), np.max(s_list_5), np.max(s_list_50), np.max(s_list_95))
    yticks(np.arange(int(axis_min_y),int(axis_max_y)+10., 10.))

    legend(loc=0)

    figurename = '/UE_analysis/m_c_biased_unbiased/m_and_c_and_prob_for_UE_sim_theo_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general
        
    savefig(params.figurepath_default + figurename + '.pdf')
#    savefig(params.figurepath_default + figurename + '.eps')
    plt.savefig(params.figurepath_default + figurename+ '.svg', format="svg")
    os.system('inkscape --without-gui --export-eps='+params.figurepath_default + figurename+'.eps '+params.figurepath_default + figurename+'.svg')
    
    
    ##########################
    ### Two-panel version: ###
    ##########################
    n_horz_panels = 2
    n_vert_panels = 1
    
    Tafel_Fabrik = create_fig_plos(figure_start_number + 10, scale, width, n_horz_panels, n_vert_panels)    

    rcParams['axes.labelsize']  = scale*10
        
    Tafel_Fabrik.new_panel(0, 0, r'A')
    plb.plot(t_sim_list, c_sim_list*10**5, label='sim.', color=black)
    plb.plot(t_theo_list, c_theo_list*10**5, label='lin. theo.', color=midgray)
    #plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$c_{ee}[10^{-5}]$')
    axis_min_y = min(np.min(c_sim_list*10**5), np.min(c_theo_list*10**5))
    axis_max_y = min(np.max(c_sim_list*10**5), np.max(c_theo_list*10**5))
    yticks(np.arange(int(axis_min_y*10.)/10.,int(axis_max_y*10.)/10.+0.2, 0.2))
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    legend(loc=0)    
    
    Tafel_Fabrik.new_panel(1, 0, r'B')
    plb.plot(t_sim_list, UE_prob_sim_list, label='sim.', color=black)
    plb.plot(t_theo_list, UE_prob_theo_list, label='lin. theo.', color=midgray)
    
    #plb.plot(t_sim_list, UE_prob_sim_lin_list, '--', label='sim.,lin.', color=black)
    #plb.plot(t_theo_list, UE_prob_theo_lin_list, '--', label='lin. theo., lin.', color=midgray)

    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\lambda}^{\mathrm{UE}}$')
    axis_min_y = min(np.min(UE_prob_sim_list), np.min(UE_prob_theo_list), np.min(UE_prob_sim_lin_list), np.min(UE_prob_theo_lin_list))
    axis_max_y = max(np.max(UE_prob_sim_list), np.max(UE_prob_theo_list), np.max(UE_prob_sim_lin_list), np.max(UE_prob_theo_lin_list))
    yticks(np.arange(int(axis_min_y*1000.)/1000.,int(axis_max_y*1000.)/1000.+0.001, 0.001))    
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    rcParams['legend.fontsize'] = scale*6
    legend(loc=0)
    

    figurename = '/UE_analysis/m_c_biased_unbiased/m_and_c_and_prob_for_UE_sim_theo_two_panels_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general
        
    savefig(params.figurepath_default + figurename + '.pdf')
    savefig(params.figurepath_default + figurename + '.eps')

    ###########################
    ### Four-panel version: ###    
    ###########################

    n_horz_panels = 2
    n_vert_panels = 2
    
    Tafel_Fabrik = create_fig_plos(figure_start_number + 11, scale, width, n_horz_panels, n_vert_panels)    
    
    
    rcParams['legend.fontsize'] = scale*8
    rcParams['axes.labelsize']  = scale*10
        
    
    Tafel_Fabrik.new_panel(0, 0, r'A')
    plb.plot(t_sim_list, c_sim_list*10**5, label='sim.', color=black)
    plb.plot(t_theo_list, c_theo_list*10**5, label='lin. theo.', color=midgray)
    #plb.plot(t_list, (np.imag(C_ee_theo * np.exp(1j * omega * t_list/1000.)) + cs[0])*10**5, label='$c_{ee, \mathrm{theo.}}(t)$')
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$c_{ee}[10^{-5}]$')
    axis_min_y = min(np.min(c_sim_list*10**5), np.min(c_theo_list*10**5))
    axis_max_y = min(np.max(c_sim_list*10**5), np.max(c_theo_list*10**5))
    yticks(np.arange(int(axis_min_y*10.)/10.,int(axis_max_y*10.)/10.+0.2, 0.2))
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    legend(loc=0)    
    
    Tafel_Fabrik.new_panel(1, 0, r'B')
    plb.plot(t_sim_list, m_sim_list*10**2, label='sim', color=black)
    plb.plot(t_theo_list, m_theo_list*10**2, label='lin. theo.', color=midgray)
    #plb.plot(t_list, (np.imag(dev_mean_freq[0] * np.exp(1j * omega * t_list/1000.)) + ms[0])*10**2, label='$m_{e, \mathrm{theo.}}(t)$')
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$m_{e}[10^{-2}]$')
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
        
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(0, 1, r'C')
    plb.plot(t_sim_list, UE_prob_sim_list, label='sim.', color=black)
    plb.plot(t_theo_list, UE_prob_theo_list, label='lin. theo.', color=midgray)
    
    plb.plot(t_sim_list, UE_prob_sim_lin_list, '--', label='sim.,lin.', color=black)
    plb.plot(t_theo_list, UE_prob_theo_lin_list, '--', label='lin. theo., lin.', color=midgray)

    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\lambda}^{\mathrm{UE}}$')
    axis_min_y = min(np.min(UE_prob_sim_list), np.min(UE_prob_theo_list), np.min(UE_prob_sim_lin_list), np.min(UE_prob_theo_lin_list))
    axis_max_y = max(np.max(UE_prob_sim_list), np.max(UE_prob_theo_list), np.max(UE_prob_sim_lin_list), np.max(UE_prob_theo_lin_list))
    yticks(np.arange(int(axis_min_y*1000.)/1000.,int(axis_max_y*1000.)/1000.+0.001, 0.001))    
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    legend(loc=0)
    
    Tafel_Fabrik.new_panel(1, 1, r'D')
    plb.plot(t_sim_list, UE_prob_biased_sim_list, label='sim., const. mean', color=black)
    plb.plot(t_theo_list, UE_prob_biased_theo_list, label='lin. theo., const. mean', color=midgray)    
    plb.xlabel(r'$\frac{\varphi}{2\pi}$')
    plb.ylabel(r'$P_{\overline{\lambda}}^{\mathrm{UE}}$ ($\mathrm{constant} \ n_{0}$)')
    axis_min_y = min(np.min(UE_prob_biased_sim_list), np.min(UE_prob_biased_theo_list))
    axis_max_y = max(np.max(UE_prob_biased_sim_list), np.max(UE_prob_biased_theo_list))
    yticks(np.arange(int(axis_min_y*10.)/10.,int(axis_max_y*10.)/10.+0.1, 0.1))
    #yticks(np.array([0.,0.1,0.2,0.25]))
    #plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    
    legend(loc=0)

    figurename = '/UE_analysis/m_c_biased_unbiased/m_and_c_and_prob_for_UE_sim_theo_four_panels_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general
        
    savefig(params.figurepath_default + figurename + '.pdf')
#    savefig(params.figurepath_default + figurename + '.eps')
    plt.savefig(params.figurepath_default + figurename+ '.svg', format="svg")
    os.system('inkscape --without-gui --export-eps='+params.figurepath_default + figurename+'.eps '+params.figurepath_default + figurename+'.svg')           

def plot_mean_surprise(m_list, c_list, h0, omega, T, figure_start_number, suffix_general):
    t_list            = np.linspace(0,2.*np.pi * 1000./omega, len(m_list), endpoint=False)
    n0_corr           = (m_list**2 + c_list)*T
    n0_uncorr         = m_list**2 *T
    mean_surpr_corr       = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_corr)
    mean_surpr_corr_minus = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_corr - np.sqrt(n0_corr))
    mean_surpr_corr_plus  = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_corr + np.sqrt(n0_corr))
    
    mean_surpr_uncorr       = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_uncorr)    
    mean_surpr_uncorr_minus = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_uncorr - np.sqrt(n0_uncorr))
    mean_surpr_uncorr_plus  = prob_UE_continuous(m_list,np.zeros_like(m_list),T,n0_uncorr + np.sqrt(n0_uncorr))
    
    plb.figure(figure_start_number + 1)
    plb.plot(t_list, mean_surpr_uncorr, color='blue', label='uncorrelated')
    plb.plot(t_list, mean_surpr_uncorr_minus, '.', color='blue')    
    plb.plot(t_list, mean_surpr_uncorr_plus, '.', color='blue')
    
    plb.plot(t_list, mean_surpr_corr, color='red', label='correlated')
    plb.plot(t_list, mean_surpr_corr_minus,'.', color='red', label='correlated')
    plb.plot(t_list, mean_surpr_corr_plus,'.', color='red', label='correlated')    
    plb.xlabel('$t [ms]$')
    plb.ylabel('Average surprise')
    plb.title('$\omega=$' + str(omega/(2.*np.pi)) + '$\cdot 2\pi$')
    plb.legend(loc=0)

    figurename = '/UE_analysis/Mean_surprise_h0='+str(h0)+'_T='+str(params.T)+ '_omega=' + str(omega) + suffix_general        
        
    plb.savefig(params.figurepath_default + figurename + '.pdf')
    plb.savefig(params.figurepath_default + figurename + '.eps')       
    
    
##def n0_finder_continuous(m,T,p0):
##    gma = m**2*T    
##    return sps.gammaincinv(p0,gma) -1
#    
##def prob_UE_continuous(m,c,T,n0):
##    prob = 1 - gamma.cdf(n0+1, (m**2+c)*T)
##    return prob
##    
##def n0_finder_continuous(m,T,p0):
##    return gamma.isf(p0,m**2*T)-1    
#    
#N_pop = 1000000
#m = 0.2
#c = 0.001
#T = 1000
#mu = m**2 * T
#
#psi = prob_UE_corr_network_for_uncorr_network_expected(m,c,T,53)    
#
#n1 = int(mu)
#n2 = int(2.*mu)
#
#n0_list = np.arange(n1,n2)
#
#prob_UE_uncorr_list                 = np.zeros(len(n0_list), dtype=float)
#prob_UE_corr_list                   = np.zeros_like(prob_UE_uncorr_list)
#prob_UE_corr_first_order_corr_list  = np.zeros(len(n0_list), dtype=float)
#prob_UE_corr_second_order_corr_list = np.zeros(len(n0_list), dtype=float)
#for ii in range(n1,n2):
##    prob_UE_uncorr_list[ii-n1]                 = prob_UE_corr_network_for_uncorr_network_expected(m,0.,T,ii)
##    prob_UE_corr_list[ii-n1]                   = prob_UE_corr_network_for_uncorr_network_expected(m,c,T,ii)
#    prob_UE_uncorr_list[ii-n1]                 = prob_UE_continuous(m,0.,T,ii)
#    prob_UE_corr_list[ii-n1]                   = prob_UE_continuous(m,c,T,ii)
##    prob_UE_corr_first_order_corr_list[ii-n1]  = prob_UE_corr_network_for_uncorr_network_expected(m,0.,T,ii) + poisson.pmf(ii-1,m**2*T) * T * c  
##    prob_UE_corr_second_order_corr_list[ii-n1] = prob_UE_corr_first_order_corr_list[ii-n1] + (-poisson.pmf(ii-1,m**2*T)+poisson.pmf(ii-2,m**2*T)) * (T * c)**2/2.
#plb.figure(1)
#plb.plot(n0_list, prob_UE_uncorr_list, '.', label='uncorrelated')
#plb.plot(n0_list, prob_UE_corr_list, '.', label='correlated, full')
##plb.plot(n0_list, prob_UE_corr_first_order_corr_list, '.', label='correlated, first order Taylor')
##plb.plot(n0_list, prob_UE_corr_second_order_corr_list, '.', label='correlated, second order Taylor')
#plb.plot(n0_list, np.ones_like(prob_UE_corr_list) * 0.05)
#plb.legend(loc=0)
#    
#print 'mu=', mu
#
#n0 = n0_finder(m,T,0.05)
#
#print 'n0=', n0
#
#del_m = 0.1 * m
##del_m = 0.
#
#del_c = 0.1 * c
#phase_c = 2. * np.pi/10.
#
#t_list = np.arange(0.,2*np.pi,0.1)
#m_list = m + del_m * np.sin(t_list)
#c_list = c + del_c * np.sin(t_list + phase_c)
#
#
#sigma_m_list = np.sqrt(m_list * (1.-m_list)/N_pop)
#sigma_c_list = np.sqrt(c_list * (1.-c_list)/N_pop)
#
#m_noise = np.random.normal(0.,sigma_m_list)
##c_noise = np.random.normal(0.,sigma_c_list)
#
##print 'Ein bisschen noise:', m_noise[:10]
#
##m_list = m_list + m_noise
##c_list = c_list + cnois
#
#plb.figure(2)
#plb.plot(t_list, m_list)
#
#plb.figure(3)
#plb.plot(t_list, c_list)
#
#n0_list = np.zeros_like(m_list)
#UE_prob_list = np.zeros_like(m_list)
#
#
#N_T = 10
#
#UE_prob_list_list = np.zeros((len(m_list),N_T), dtype=float)
#
#for jj in range(0,N_T):
#    T = T+1
#    print 'T=', T
##    for ii in range(0,len(n0_list)):
##        #n0               = n0_finder(m_list[ii],T,0.05)
##        n0               = poisson_continuous_inv(m_list[ii],T,0.05)        
##        n0_list[ii]      = n0
##        #UE_prob_list[ii] = prob_UE_corr_network_for_uncorr_network_expected(m_list[ii],c_list[ii],T,n0)
##        UE_prob_list[ii] = prob_UE_continuous(m_list[ii],c_list[ii],T,n0)
##        
#
#    n0_list, UE_prob_list  = UE_prob(m_list, c_list, T, 0.05)
#
#    UE_prob_list_list[:,jj] = UE_prob_list 
#    
#    #print "Fertig mit n0_list"
#        
#    plb.figure(4)
#    plb.plot(t_list, n0_list)
#    
#    plb.figure(5)
#    plb.plot(t_list, UE_prob_list)
#    
#    UE_prob_FT = np.fft.fft(UE_prob_list)
#    freq       = np.fft.fftfreq(len(UE_prob_list))
#    
#    plb.figure(6)
#    plb.plot(freq, np.abs(UE_prob_FT), '.')
#
#UE_prob_list_average = np.mean(UE_prob_list_list, axis = 1)
#UE_prob_FT_average   = np.fft.fft(UE_prob_list_average)
#
#plb.figure(7)
#plb.plot(t_list, UE_prob_list_average)
#
#plb.figure(8)
#plb.plot(freq, np.abs(UE_prob_FT_average), '.')
#
#
##Surprise as a function of time
#
##surprise_list = np.zeros_like(c_list)
##
##for ii in range(0,len(c_list)):
##    surprise_list[ii]=prob_UE_corr_network_for_uncorr_network_expected(m_list[ii],0.,T,int((m_list[ii]**2+c_list[ii])*T))
##    
##plb.figure(9)
##plb.plot(t_list, surprise_list)
#
##Incoplete Gamma function!!!!!! gamma_inc in scipy special