# -*- coding: utf-8 -*-
"""
Created on Tue Feb  3 11:38:43 2015

@author: kuehn
"""

import numpy as np

E = 0
I = 1
X = 2

EE = 0
EI = 1
II = 2
EX = 3
IX = 4

E1 = 0
E2 = 1
I1 = 2
I2 = 3

class compute:
    
    def __init__(self, m_exc1, m_exc2, m_inh1, m_inh2, tau_m, dt_samp):
        self.m_exc1  = m_exc1
        self.m_exc2  = m_exc2
        self.m_inh1  = m_inh1
        self.m_inh2  = m_inh2
        self.tau_m   = tau_m
        self.dt_samp = dt_samp
        
    def compute_corr_ts_directly(self, N_one_period):
        """Calculates the zero time-lag covariances of the excitatory/inhibitory system
        in time space. For this, we assume that every period (i.e. N_one_period successive data points)
        is a trial and the trials are not correlated."""
        #N_transient = int(100.*self.tau_m/self.dt_samp)
        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period
        #N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    #Approximate the transient phase to last for 5*tau_m
        N_ges = len(self.m_exc1)
        N_number_periods = int((N_ges-N_transient)/N_one_period)
    #    N_eff = N_number_periods*N_one_period
        m_sorted      = np.zeros((N_number_periods, N_one_period,4), dtype=float)   #Sorted according to  affiliation to a period
      
        for r in range(0,N_number_periods):                                         #We interprete each period as one realization of the ensemble
            m_sorted[r,:,E1] = self.m_exc1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]    
            m_sorted[r,:,E2] = self.m_exc2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
            m_sorted[r,:,I1] = self.m_inh1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]   
            m_sorted[r,:,I2] = self.m_inh2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
        
        mean_per = np.zeros((N_number_periods,N_one_period,4), dtype=float)      #"per" is for "(one) period" or "per period"    
        for s in range(0,N_one_period):
            for t in range(0,4):
                mean_per[:,s,t]=np.mean(m_sorted[:,s,t])
        
        c_quer       = np.zeros((3,N_one_period), dtype=complex)
        c_quer[EE,:] = np.mean((m_sorted[:,:,E1] - mean_per[:,:,E1]) * (m_sorted[:,:,E2] - mean_per[:,:,E2]), axis=0)
        c_quer[II,:] = np.mean((m_sorted[:,:,I1] - mean_per[:,:,I1]) * (m_sorted[:,:,I2] - mean_per[:,:,I2]), axis=0)
        c_quer[EI,:] = np.mean((m_sorted[:,:,I1] - mean_per[:,:,I1]) * (m_sorted[:,:,E1] - mean_per[:,:,E1]), axis=0)
        
        return mean_per[0,:,:], c_quer
        
    
    def Fourier_corr_quer(self, N_one_period, count):
        """Calculates the zero time-lag covariances of the excitatory/inhibitory system
        in frequency space. For this, we assume that every period (i.e. N_one_period 
        successive data points) is a trial and the trials are not correlated.
        Practically it calculates fft(AB) over the whole time stretch, picks out
        only the frequencies which are integer multiples of 2*pi/T (T the period
        of the perturbation) and substracts A_hat_bar * B_hat_bar, where 
        A_hat_bar and B_hat_bar are the fft of A and B respectively with the coefficients
        belonging to integer multiples of 2*pi/T picked out."""
    #    N_transient = int(10.*tau_m/dt_samp)  #Approximate the transient phase to last for 10*tau_m
#        N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    
        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period        
        #print 'N_transient=', N_transient
        N_ges = len(self.m_exc1)
        #print 'N_ges=', N_ges
        
        N_number_periods = int((N_ges-N_transient)/N_one_period)    
        N_eff            = N_number_periods * N_one_period
        print 'N_ges='           , N_ges 
        print 'N_number_periods=', N_number_periods    
        print 'N_eff='           , N_eff
        m_exc1_eff = self.m_exc1[N_transient:N_transient + N_eff]
        m_exc2_eff = self.m_exc2[N_transient:N_transient + N_eff]
        m_inh1_eff = self.m_inh1[N_transient:N_transient + N_eff]
        m_inh2_eff = self.m_inh2[N_transient:N_transient + N_eff]
    
        mean_exc1 = np.mean(m_exc1_eff)
        mean_exc2 = np.mean(m_exc2_eff)
        mean_inh1 = np.mean(m_inh1_eff)
        mean_inh2 = np.mean(m_inh2_eff)      
        
        nn_ee_quer = np.fft.fft((m_exc1_eff - mean_exc1) * (m_exc2_eff - mean_exc2))/N_eff
        nn_ii_quer = np.fft.fft((m_inh1_eff - mean_inh1) * (m_inh2_eff - mean_inh2))/N_eff
        nn_ei_quer = np.fft.fft((m_inh1_eff - mean_inh1) * (m_exc1_eff - mean_exc1))/N_eff
        
        nn_ee_quer_periodical_mean = nn_ee_quer [np.arange(0,N_one_period) * N_number_periods] 
        nn_ii_quer_periodical_mean = nn_ii_quer [np.arange(0,N_one_period) * N_number_periods] 
        nn_ei_quer_periodical_mean = nn_ei_quer [np.arange(0,N_one_period) * N_number_periods]     
    
        me_f1=np.fft.fft(m_exc1_eff - np.mean(m_exc1_eff))/N_eff
        me_f2=np.fft.fft(m_exc2_eff - np.mean(m_exc2_eff))/N_eff
        mi_f1=np.fft.fft(m_inh1_eff - np.mean(m_inh1_eff))/N_eff
        mi_f2=np.fft.fft(m_inh2_eff - np.mean(m_inh2_eff))/N_eff
       
        me_f1_periodical_mean = me_f1[np.arange(0,N_one_period) * N_number_periods]        # "Average" the Fourier signal by picking out only the coefficients 
        mi_f1_periodical_mean = mi_f1[np.arange(0,N_one_period) * N_number_periods]        #suiting to a periodical function with period T_ext        
        me_f2_periodical_mean = me_f2[np.arange(0,N_one_period) * N_number_periods]        
        mi_f2_periodical_mean = mi_f2[np.arange(0,N_one_period) * N_number_periods]    
    
        ne_ne_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)
        ni_ni_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)
        ne_ni_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)    
        
        for k in range(0,N_one_period):
            summe_ee = 0.
            summe_ii = 0.
            summe_ei = 0.        
            for l in range(k-(N_one_period-1),k+1):
                if(l >= 0):
                    l_index = l
                else:
                    l_index = l+N_one_period
                summe_ee = summe_ee +  me_f1_periodical_mean[l_index] * me_f2_periodical_mean[k-l]        
                summe_ii = summe_ii +  mi_f1_periodical_mean[l_index] * mi_f2_periodical_mean[k-l]
                summe_ei = summe_ei +  me_f1_periodical_mean[l_index] * mi_f1_periodical_mean[k-l]
            ne_ne_quer_periodical_mean[k]= summe_ee
            ni_ni_quer_periodical_mean[k]= summe_ii
            ne_ni_quer_periodical_mean[k]= summe_ei
            #print 'Faltungsregel vermutlich korrekt.'
        
        freq =  np.fft.fftfreq(N_one_period, d=self.dt_samp)*1000
        
        c_roh       = np.zeros((3,N_one_period), dtype = complex)
        c_roh[EE,:] = nn_ee_quer_periodical_mean #- ne_ne_quer_periodical_mean
        c_roh[II,:] = nn_ii_quer_periodical_mean #- ni_ni_quer_periodical_mean
        c_roh[EI,:] = nn_ei_quer_periodical_mean #- ne_ni_quer_periodical_mean
        
        c_naiv       = np.zeros((3,N_one_period), dtype = complex)
        c_naiv[EE,:] = ne_ne_quer_periodical_mean
        c_naiv[II,:] = ni_ni_quer_periodical_mean
        c_naiv[EI,:] = ne_ni_quer_periodical_mean
    
        return freq, c_roh, c_naiv
        
    
        
    def Fourier_corr_quer_indie(self, N_one_period, count):
        """Calculates the zero time-lag covariances of the excitatory/inhibitory system
        in frequency space. For this, we assume that every period (i.e. N_one_period 
        successive data points) is a trial and the trials are not correlated.
        It divides the data in two halfs and calculates the mean acitivies A_bar and B_bar 
        of A and B respectivelz of the first half and computes then the mean of the quantity
        (A - A_bar) * (B - B_bar) over the second half.
        Means are always calculated by computing the fft and then throwing away all contributions
        of frequencies that are no integer multiples of 2*pi/T."""
    #    N_transient = int(10.*tau_m/dt_samp)  #Approximate the transient phase to last for 10*tau_m
#        N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period
        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period
        #print 'N_transient=', N_transient
        N_ges = len(self.m_exc1)
        #print 'N_ges=', N_ges
        
        N_number_periods = int((N_ges-N_transient)/N_one_period)
        N_number_periods_first = N_number_periods/2    
        N_number_periods_second = N_number_periods - N_number_periods_first
        
        N_eff                  = N_number_periods * N_one_period    
        N_eff_first            = N_number_periods_first * N_one_period
        N_eff_second           = N_number_periods_second * N_one_period
        
    #    m_exc1_eff = self.m_exc1[N_transient:N_transient + N_eff]
    #    m_exc2_eff = self.m_exc2[N_transient:N_transient + N_eff]
    #    m_inh1_eff = self.m_inh1[N_transient:N_transient + N_eff]
    #    m_inh2_eff = self.m_inh2[N_transient:N_transient + N_eff]
        
        m_exc1_eff_first = self.m_exc1[N_transient:N_transient + N_eff_first]
        m_exc2_eff_first = self.m_exc2[N_transient:N_transient + N_eff_first]
        m_inh1_eff_first = self.m_inh1[N_transient:N_transient + N_eff_first]
        m_inh2_eff_first = self.m_inh2[N_transient:N_transient + N_eff_first]
    
        m_exc1_eff_second = self.m_exc1[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
        m_exc2_eff_second = self.m_exc2[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
        m_inh1_eff_second = self.m_inh1[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
        m_inh2_eff_second = self.m_inh2[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
      
    #####################################################################################################
    ### That works: #####################################################################################
    #####################################################################################################  
      
    #    me_f1  = np.fft.fft(m_exc1_eff)/N_eff
    #    me_f2  = np.fft.fft(m_exc2_eff)/N_eff
    #    mi_f1  = np.fft.fft(m_inh1_eff)/N_eff
    #    mi_f2  = np.fft.fft(m_inh2_eff)/N_eff
        
        me_f1_first  = np.fft.fft(m_exc1_eff_first)/N_eff_first
        me_f2_first  = np.fft.fft(m_exc2_eff_first)/N_eff_first
        mi_f1_first  = np.fft.fft(m_inh1_eff_first)/N_eff_first
        mi_f2_first  = np.fft.fft(m_inh2_eff_first)/N_eff_first
        
        me_f1_periodical_mean_first = me_f1_first[np.arange(0,N_one_period) * N_number_periods_first]        # "Average" the Fourier signal by picking out only the coefficients 
        mi_f1_periodical_mean_first = mi_f1_first[np.arange(0,N_one_period) * N_number_periods_first]        #suiting to a periodical function with period T_ext        
        me_f2_periodical_mean_first = me_f2_first[np.arange(0,N_one_period) * N_number_periods_first]        
        mi_f2_periodical_mean_first = mi_f2_first[np.arange(0,N_one_period) * N_number_periods_first]
    
        me_f1_periodical_mean_long_second  = np.zeros((N_one_period, N_number_periods_second), dtype=complex)        # "Average" the Fourier signal by picking out only the coefficients 
        mi_f1_periodical_mean_long_second  = np.zeros((N_one_period, N_number_periods_second), dtype=complex)        #suiting to a periodical function with period T_ext        
        me_f2_periodical_mean_long_second  = np.zeros((N_one_period, N_number_periods_second), dtype=complex)        
        mi_f2_periodical_mean_long_second  = np.zeros((N_one_period, N_number_periods_second), dtype=complex)
        
        me_f1_periodical_mean_long_second[:,0] = me_f1_periodical_mean_first
        mi_f1_periodical_mean_long_second[:,0] = mi_f1_periodical_mean_first
        me_f2_periodical_mean_long_second[:,0] = me_f2_periodical_mean_first  
        mi_f2_periodical_mean_long_second[:,0] = mi_f2_periodical_mean_first
        
        me_f1_periodical_mean_long_second = me_f1_periodical_mean_long_second.flatten()
        mi_f1_periodical_mean_long_second = mi_f1_periodical_mean_long_second.flatten()
        me_f2_periodical_mean_long_second = me_f2_periodical_mean_long_second.flatten()
        mi_f2_periodical_mean_long_second = mi_f2_periodical_mean_long_second.flatten()
        
        print 'me_f1_periodical_mean_long_second.shape = ', me_f1_periodical_mean_long_second.shape
    
        nn_ee_quer = np.fft.fft((m_exc1_eff_second - np.fft.ifft(me_f1_periodical_mean_long_second)*N_eff_second) * (m_exc2_eff_second - np.fft.ifft(me_f2_periodical_mean_long_second)*N_eff_second))/N_eff_second
        nn_ii_quer = np.fft.fft((m_inh1_eff_second - np.fft.ifft(mi_f1_periodical_mean_long_second)*N_eff_second) * (m_inh2_eff_second - np.fft.ifft(mi_f2_periodical_mean_long_second)*N_eff_second))/N_eff_second
        nn_ei_quer = np.fft.fft((m_inh1_eff_second - np.fft.ifft(mi_f1_periodical_mean_long_second)*N_eff_second) * (m_exc1_eff_second - np.fft.ifft(mi_f1_periodical_mean_long_second)*N_eff_second))/N_eff_second
        
        nn_ee_quer_periodical_mean = nn_ee_quer [np.arange(0,N_one_period) * N_number_periods_second] 
        nn_ii_quer_periodical_mean = nn_ii_quer [np.arange(0,N_one_period) * N_number_periods_second] 
        nn_ei_quer_periodical_mean = nn_ei_quer [np.arange(0,N_one_period) * N_number_periods_second]
        
        c_roh       = np.zeros((3,N_one_period), dtype=complex)
        c_roh[EE,:] = nn_ee_quer_periodical_mean
        c_roh[II,:] = nn_ii_quer_periodical_mean
        c_roh[EI,:] = nn_ei_quer_periodical_mean
        
        c_naiv = np.zeros((3,N_one_period), dtype=complex)
        
        freq =  np.fft.fftfreq(N_one_period, d=self.dt_samp)*1000
        
        return freq, c_roh, c_naiv
    
    def Fourier_corr_quer_chopped(self, N_one_period, count):
        """This function calculates the correlations of exc-exc, exc-inh and inh-inh neurons.
        It assumes that the data of periods that are separated at least by the transient time
        are uncorrelated and can therefore be regarded as realizations of the same ensemble.
        Averages are always computed by calculating the fft and throwing away all contributions belonging
        to frequencies that are no muktiples of 2*pi/T.  """
    #    N_transient = int(10.*tau_m/dt_samp)  #Approximate the transient phase to last for 10*tau_m
#        N_transient = int(int(10.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period
        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period
        #N_transient = 3
        #print 'N_transient=', N_transient
        N_ges = len(self.m_exc1)
        #print 'N_ges=', N_ges
        
        N_number_periods = int((N_ges-N_transient)/N_one_period)
        
        N_eff                  = N_number_periods * N_one_period    
        N_periods_transient    = (int(N_transient/N_one_period)+1)              #N_periods_transient * T > t_transient
    
        m_exc1_eff = self.m_exc1[N_transient:N_transient + N_eff]
        m_exc2_eff = self.m_exc2[N_transient:N_transient + N_eff]
        m_inh1_eff = self.m_inh1[N_transient:N_transient + N_eff]
        m_inh2_eff = self.m_inh2[N_transient:N_transient + N_eff]
        
        m_exc1_eff_sorted = np.zeros((N_number_periods,N_one_period), dtype=complex)
        m_exc2_eff_sorted = np.zeros((N_number_periods,N_one_period), dtype=complex)
        m_inh1_eff_sorted = np.zeros((N_number_periods,N_one_period), dtype=complex)
        m_inh2_eff_sorted = np.zeros((N_number_periods,N_one_period), dtype=complex)
        
        for ii in range(0,N_one_period):
            m_exc1_eff_sorted[:,ii] = m_exc1_eff [np.arange(0,N_number_periods) * N_one_period + ii]
            m_exc2_eff_sorted[:,ii] = m_exc2_eff [np.arange(0,N_number_periods) * N_one_period + ii]
            m_inh1_eff_sorted[:,ii] = m_inh1_eff [np.arange(0,N_number_periods) * N_one_period + ii]
            m_inh2_eff_sorted[:,ii] = m_inh2_eff [np.arange(0,N_number_periods) * N_one_period + ii]
        
        chose_periods = np.arange(0,N_number_periods,N_periods_transient+1)
        
        N_number_periods = len(chose_periods)                   #Mind the redefinitions!
        N_eff            = N_number_periods * N_one_period    
        
        m_exc1_eff_chosen = m_exc1_eff_sorted[chose_periods,:]
        m_exc2_eff_chosen = m_exc2_eff_sorted[chose_periods,:] 
        m_inh1_eff_chosen = m_inh1_eff_sorted[chose_periods,:]
        m_inh2_eff_chosen = m_inh2_eff_sorted[chose_periods,:]
        
        m_exc1_eff = m_exc1_eff_chosen.flatten()            
        m_exc2_eff = m_exc2_eff_chosen.flatten()
        m_inh1_eff = m_inh1_eff_chosen.flatten()
        m_inh2_eff = m_inh2_eff_chosen.flatten()
            
        mean_exc1 = np.mean(m_exc1_eff)
        mean_exc2 = np.mean(m_exc2_eff)
        mean_inh1 = np.mean(m_inh1_eff)
        mean_inh2 = np.mean(m_inh2_eff)      
        
        nn_ee_quer = np.fft.fft((m_exc1_eff - mean_exc1) * (m_exc2_eff - mean_exc2))/N_eff
        nn_ii_quer = np.fft.fft((m_inh1_eff - mean_inh1) * (m_inh2_eff - mean_inh2))/N_eff
        nn_ei_quer = np.fft.fft((m_inh1_eff - mean_inh1) * (m_exc1_eff - mean_exc1))/N_eff
        
        nn_ee_quer_periodical_mean = nn_ee_quer [np.arange(0,N_one_period) * N_number_periods] 
        nn_ii_quer_periodical_mean = nn_ii_quer [np.arange(0,N_one_period) * N_number_periods] 
        nn_ei_quer_periodical_mean = nn_ei_quer [np.arange(0,N_one_period) * N_number_periods]     
    
        me_f1=np.fft.fft(m_exc1_eff - np.mean(m_exc1_eff))/N_eff
        me_f2=np.fft.fft(m_exc2_eff - np.mean(m_exc2_eff))/N_eff
        mi_f1=np.fft.fft(m_inh1_eff - np.mean(m_inh1_eff))/N_eff
        mi_f2=np.fft.fft(m_inh2_eff - np.mean(m_inh2_eff))/N_eff
       
        me_f1_periodical_mean = me_f1[np.arange(0,N_one_period) * N_number_periods]        # "Average" the Fourier signal by picking out only the coefficients 
        mi_f1_periodical_mean = mi_f1[np.arange(0,N_one_period) * N_number_periods]        #suiting to a periodical function with period T_ext        
        me_f2_periodical_mean = me_f2[np.arange(0,N_one_period) * N_number_periods]        
        mi_f2_periodical_mean = mi_f2[np.arange(0,N_one_period) * N_number_periods]    
    
        ne_ne_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)
        ni_ni_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)
        ne_ni_quer_periodical_mean = np.zeros(N_one_period, dtype=complex)    
        
        for k in range(0,N_one_period):
            summe_ee = 0.
            summe_ii = 0.
            summe_ei = 0.        
            for l in range(k-(N_one_period-1),k+1):
                if(l >= 0):
                    l_index = l
                else:
                    l_index = l+N_one_period
                summe_ee = summe_ee +  me_f1_periodical_mean[l_index] * me_f2_periodical_mean[k-l]        
                summe_ii = summe_ii +  mi_f1_periodical_mean[l_index] * mi_f2_periodical_mean[k-l]
                summe_ei = summe_ei +  me_f1_periodical_mean[l_index] * mi_f1_periodical_mean[k-l]
            ne_ne_quer_periodical_mean[k]= summe_ee
            ni_ni_quer_periodical_mean[k]= summe_ii
            ne_ni_quer_periodical_mean[k]= summe_ei
            #print 'Faltungsregel vermutlich korrekt.'
        
        freq =  np.fft.fftfreq(N_one_period, d=self.dt_samp)*1000
        
        c_roh       = np.zeros((3,N_one_period), dtype = complex)
        c_roh[EE,:] = nn_ee_quer_periodical_mean #- ne_ne_quer_periodical_mean
        c_roh[II,:] = nn_ii_quer_periodical_mean #- ni_ni_quer_periodical_mean
        c_roh[EI,:] = nn_ei_quer_periodical_mean #- ne_ni_quer_periodical_mean
        
        c_naiv       = np.zeros((3,N_one_period), dtype = complex)
        c_naiv[EE,:] = ne_ne_quer_periodical_mean
        c_naiv[II,:] = ni_ni_quer_periodical_mean
        c_naiv[EI,:] = ne_ni_quer_periodical_mean
    
        return freq, c_roh, c_naiv
        
    def const_corr(self, N_one_period):
        """Computes the constant part of the correlations by assuming that every period
        (consisting of N_one_period data points) is a independent trial. This is different
        from calculating the mean over the whole time stretch and computing the mean deviation
        of that!"""
#        if(N_one_period==1):
#            print 'Urspruengliche Methode wird angewendet.'
#            N_transient = int(100.*self.tau_m/self.dt_samp)    #Approximate the transient phase to last for 5*tau_m
#            self.m_exc1    = self.m_exc1[N_transient:]
#            self.m_exc2    = self.m_exc2[N_transient:]            
#            self.m_inh1    = self.m_inh1[N_transient:]
#            self.m_inh2    = self.m_inh2[N_transient:]             
#            mean_exc1 = np.mean(self.m_exc1)
#            mean_inh1 = np.mean(self.m_inh1)
#            mean_exc2 = np.mean(self.m_exc2)
#            mean_inh2 = np.mean(self.m_inh2)
#            c_quer_const       = np.zeros(3, dtype=complex)
##            c_quer_const[EE] = (1./(len(self.m_exc1)) * np.correlate(self.m_exc1-mean_exc1, self.m_exc2-mean_exc2, mode='full'))[len(self.m_exc1)]
##            c_quer_const[EI] = (1./(len(self.m_exc1)) * np.correlate(self.m_exc1-mean_exc1, self.m_inh1-mean_inh1, mode='full'))[len(self.m_exc1)]
##            c_quer_const[II] = (1./(len(self.m_exc1)) * np.correlate(self.m_inh1-mean_inh1, self.m_inh2-mean_inh2, mode='full'))[len(self.m_exc1)]
#            
#            
#            c_quer_const[EE] = np.mean((self.m_exc1-mean_exc1) * (self.m_exc2-mean_exc2))
#            c_quer_const[EI] = np.mean((self.m_exc1-mean_exc1) * (self.m_inh1-mean_inh1))
#            c_quer_const[II] = np.mean((self.m_inh1-mean_inh1) * (self.m_inh2-mean_inh2))
#
##            c_quer_const[EE] = np.max(1./(len(self.m_exc1)) * np.correlate(self.m_exc1-mean_exc1, self.m_exc2-mean_exc2, mode='full'))
##            c_quer_const[EI] = np.max(1./(len(self.m_exc1)) * np.correlate(self.m_exc1-mean_exc1, self.m_inh1-mean_inh1, mode='full'))
##            c_quer_const[II] = np.max(1./(len(self.m_exc1)) * np.correlate(self.m_inh1-mean_inh1, self.m_inh2-mean_inh2, mode='full'))
#        else:
        #N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    #Approximate the transient phase to last for 5*tau_m
        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period        
        N_ges = len(self.m_exc1)
        N_number_periods = int((N_ges-N_transient)/N_one_period)
        print 'N_number_periods=', N_number_periods
        #    N_eff = N_number_periods*N_one_period
        m_sorted      = np.zeros((N_number_periods, N_one_period,4), dtype=float)   #Sorted according to  affiliation to a period
          
        for r in range(0,N_number_periods):                                         #We interprete each period as one realization of the ensemble
            m_sorted[r,:,E1] = self.m_exc1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]    
            m_sorted[r,:,E2] = self.m_exc2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
            m_sorted[r,:,I1] = self.m_inh1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]   
            m_sorted[r,:,I2] = self.m_inh2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
            
        mean_per = np.zeros((N_number_periods,N_one_period,4), dtype=float)      #"per" is for "(one) period" or "per period"    
        for s in range(0,N_one_period):
            for t in range(0,4):
                mean_per[:,s,t]=np.mean(m_sorted[:,s,t])
            
        c_quer_const       = np.zeros(3, dtype=complex)
        c_quer_const[EE]   = np.mean((m_sorted[:,:,E1] - mean_per[:,:,E1]) * (m_sorted[:,:,E2] - mean_per[:,:,E2]))
        c_quer_const[II]   = np.mean((m_sorted[:,:,I1] - mean_per[:,:,I1]) * (m_sorted[:,:,I2] - mean_per[:,:,I2]))
        c_quer_const[EI]   = np.mean((m_sorted[:,:,I1] - mean_per[:,:,I1]) * (m_sorted[:,:,E1] - mean_per[:,:,E1]))
        
        return c_quer_const
    
    
    ############################################################
    ### "Vectorized" verions of some of the above functions: ###
    ############################################################
    
    #    def compute_corr_ts_directly(self, N_one_period):    
    #    
    #        #N_transient = int(100.*self.tau_m/self.dt_samp)
    #        #N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    #Approximate the transient phase to last for 5*tau_m
    #        N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period
    #        N_ges = len(self.m_exc1)
    #        N_number_periods = int((N_ges-N_transient)/N_one_period)
    #    #    N_eff = N_number_periods*N_one_period
    #        
    #        m_exc1_sorted = np.zeros((N_number_periods, N_one_period), dtype=float)     #Sorted according to  affiliation to a period     
    #        m_exc2_sorted = np.zeros((N_number_periods, N_one_period), dtype=float)
    #        m_inh1_sorted = np.zeros((N_number_periods, N_one_period), dtype=float)    
    #        m_inh2_sorted = np.zeros((N_number_periods, N_one_period), dtype=float)
    #      
    #        for r in range(0,N_number_periods):                                         #We interprete each period as one realization of the ensemble
    #            m_exc1_sorted[r,:] = self.m_exc1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]    
    #            m_exc2_sorted[r,:] = self.m_exc2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
    #            m_inh1_sorted[r,:] = self.m_inh1[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]   
    #            m_inh2_sorted[r,:] = self.m_inh2[N_transient+r*N_one_period : N_transient+(r+1)*N_one_period]
    #        
    #        mean_exc1_per = np.zeros((N_number_periods,N_one_period), dtype=float)      #"per" is for "(one) period" or "per period"
    #        mean_exc2_per = np.zeros((N_number_periods,N_one_period), dtype=float)
    #        mean_inh1_per = np.zeros((N_number_periods,N_one_period), dtype=float)
    #        mean_inh2_per = np.zeros((N_number_periods,N_one_period), dtype=float)    
    #        for s in range(0,N_one_period):
    #            mean_exc1_per[:,s]=np.mean(m_exc1_sorted[:,s])
    #            mean_exc2_per[:,s]=np.mean(m_exc2_sorted[:,s])
    #            mean_inh1_per[:,s]=np.mean(m_inh1_sorted[:,s])
    #            mean_inh2_per[:,s]=np.mean(m_inh2_sorted[:,s])
    #    
    #        c_ee_quer = np.mean((m_exc1_sorted - mean_exc1_per) * (m_exc2_sorted - mean_exc2_per), axis=0)
    #        c_ii_quer = np.mean((m_inh1_sorted - mean_inh1_per) * (m_inh2_sorted - mean_inh2_per), axis=0)
    #        c_ei_quer = np.mean((m_inh1_sorted - mean_inh1_per) * (m_exc1_sorted - mean_exc1_per), axis=0)
    #        
    #        return mean_exc1_per[0,:], mean_exc2_per[0,:], mean_inh1_per[0,:], mean_inh2_per[0,:], c_ee_quer, c_ii_quer, c_ei_quer
    #def Fourier_corr_quer(self, N_one_period, count):
    ##    N_transient = int(10.*tau_m/dt_samp)  #Approximate the transient phase to last for 10*tau_m
    #    #N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    
    #    N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period    
    #    #print 'N_transient=', N_transient
    #    N_ges = len(self.m_exc1)
    #    #print 'N_ges=', N_ges
    #    
    #    N_number_periods = int((N_ges-N_transient)/N_one_period)    
    #    N_eff            = N_number_periods * N_one_period
    #    print 'N_ges='           , N_ges 
    #    print 'N_number_periods=', N_number_periods    
    #    print 'N_eff='           , N_eff
    #    
    #    m_eff = np.zeros((4,N_eff), dtype=float)    
    #    
    #    m_eff[E1,:] = self.m_exc1[N_transient:N_transient + N_eff]
    #    m_eff[E2,:] = self.m_exc2[N_transient:N_transient + N_eff]
    #    m_eff[I1,:] = self.m_inh1[N_transient:N_transient + N_eff]
    #    m_eff[I2,:] = self.m_inh2[N_transient:N_transient + N_eff]
    #
    #    mean = np.zeros((4,N_eff), dtype=float)
    #    for ii in range(0,4):
    #        mean[ii,:] = np.mean(m_eff[ii,:])
    #    
    #    nn_quer = np.zeros((3,N_eff), dtype=complex)
    #    nn_quer[EE,:] = np.fft.fft((m_eff[E1,:] - mean[E1,:]) * (m_eff[E2,:] - mean[E2,:]))/N_eff
    #    nn_quer[EI,:] = np.fft.fft((m_eff[I1,:] - mean[I1,:]) * (m_eff[E1,:] - mean[E1,:]))/N_eff    
    #    nn_quer[II,:] = np.fft.fft((m_eff[I1,:] - mean[I1,:]) * (m_eff[I2,:] - mean[I2,:]))/N_eff    
    #    
    #    nn_quer_periodical_mean = nn_quer [:,np.arange(0,N_one_period) * N_number_periods]     
    #
    #    m_f=np.fft.fft(m_eff - mean,axis=1)/N_eff
    #   
    #    m_f_periodical_mean = m_f[:,np.arange(0,N_one_period) * N_number_periods]        # "Average" the Fourier signal by picking out only the coefficients suiting to a periodical function with period T_ext        
    #    
    #    n_n_quer_periodical_mean = np.zeros((3,N_one_period), dtype=complex)    
    #    
    #    for k in range(0,N_one_period):
    #        summe = np.zeros(3, dtype=complex)        
    #        for l in range(k-(N_one_period-1),k+1):
    #            if(l >= 0):
    #                l_index = l
    #            else:
    #                l_index = l+N_one_period
    #            summe[EE] = summe[EE] +  m_f_periodical_mean[E1,l_index] * m_f_periodical_mean[E2,k-l]        
    #            summe[EI] = summe[EI] +  m_f_periodical_mean[E1,l_index] * m_f_periodical_mean[I1,k-l]
    #            summe[II] = summe[II] +  m_f_periodical_mean[I1,l_index] * m_f_periodical_mean[I2,k-l]
    #        n_n_quer_periodical_mean[:,k]= summe
    #        #print 'Faltungsregel vermutlich korrekt.'
    #    
    #    freq =  np.fft.fftfreq(N_one_period, d=self.dt_samp)*1000
    #
    #    return freq, nn_quer_periodical_mean[EE,:], nn_quer_periodical_mean[II,:], nn_quer_periodical_mean[EI,:], n_n_quer_periodical_mean[EE,:], n_n_quer_periodical_mean[II,:], n_n_quer_periodical_mean[EI,:]
    
    #def Fourier_corr_quer_indie(self, N_one_period, count):
    ##    N_transient = int(10.*tau_m/dt_samp)  #Approximate the transient phase to last for 10*tau_m
    #    #N_transient = int(int(100.*self.tau_m/self.dt_samp)/N_one_period) * N_one_period    
    #    N_transient = (int(int(10.*self.tau_m/self.dt_samp)/N_one_period)+1) * N_one_period    
    #    #print 'N_transient=', N_transient
    #    N_ges = len(self.m_exc1)
    #    #print 'N_ges=', N_ges
    #    
    #    N_number_periods = int((N_ges-N_transient)/N_one_period)
    #    N_number_periods_first = N_number_periods/2    
    #    N_number_periods_second = N_number_periods - N_number_periods_first
    #    
    #    N_eff                  = N_number_periods * N_one_period    
    #    N_eff_first            = N_number_periods_first * N_one_period
    #    N_eff_second           = N_number_periods_second * N_one_period
    #
    #    m_eff_first       = np.zeros((4,N_eff_first), dtype=float)    
    #    m_eff_first[E1,:] = self.m_exc1[N_transient:N_transient + N_eff_first]
    #    m_eff_first[E2,:] = self.m_exc2[N_transient:N_transient + N_eff_first]
    #    m_eff_first[I1,:] = self.m_inh1[N_transient:N_transient + N_eff_first]
    #    m_eff_first[I2,:] = self.m_inh2[N_transient:N_transient + N_eff_first]
    #
    #    m_eff_second       = np.zeros((4,N_eff_second), dtype=float)
    #    m_eff_second[E1,:] = self.m_exc1[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
    #    m_eff_second[E2,:] = self.m_exc2[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
    #    m_eff_second[I1,:] = self.m_inh1[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
    #    m_eff_second[I2,:] = self.m_inh2[N_transient + N_eff_first: N_transient + N_eff_first + N_eff_second]
    #  
    ######################################################################################################
    #### That works: #####################################################################################
    ######################################################################################################  
    #  
    #    m_f_first  = np.fft.fft(m_eff_first, axis=1)/N_eff_first
    #    m_f_periodical_mean_first = m_f_first[:,np.arange(0,N_one_period) * N_number_periods_first]        # "Average" the Fourier signal by picking out only the coefficients 
    #    
    #    m_f_periodical_mean_long_second         = np.zeros((4,N_eff_second), dtype=complex)
    #    m_f_periodical_mean_long_second_matrix  = np.zeros((4,N_one_period, N_number_periods_second), dtype=complex)        # "Average" the Fourier signal by picking out only the coefficients suiting to a periodical function with period T_ext        
    #    for ii in range(0,4):    
    #        m_f_periodical_mean_long_second_matrix[ii,:,0] = m_f_periodical_mean_first[ii,:]
    #        m_f_periodical_mean_long_second[ii,:] = m_f_periodical_mean_long_second_matrix[ii,:,:].flatten()
    #        print 'me_f1_periodical_mean_long_second.shape = ', m_f_periodical_mean_long_second[E1,:].shape
    #
    #    nn_ee_quer = np.fft.fft((m_eff_second[E1,:] - np.fft.ifft(m_f_periodical_mean_long_second[E1,:])*N_eff_second) * (m_eff_second[E2,:] - np.fft.ifft(m_f_periodical_mean_long_second[E2,:])*N_eff_second))/N_eff_second
    #    nn_ii_quer = np.fft.fft((m_eff_second[I1,:] - np.fft.ifft(m_f_periodical_mean_long_second[I1,:])*N_eff_second) * (m_eff_second[I2,:] - np.fft.ifft(m_f_periodical_mean_long_second[I2,:])*N_eff_second))/N_eff_second
    #    nn_ei_quer = np.fft.fft((m_eff_second[I1,:] - np.fft.ifft(m_f_periodical_mean_long_second[I1,:])*N_eff_second) * (m_eff_second[E1,:] - np.fft.ifft(m_f_periodical_mean_long_second[E1,:])*N_eff_second))/N_eff_second
    #    
    #    nn_ee_quer_periodical_mean = nn_ee_quer [np.arange(0,N_one_period) * N_number_periods_second] 
    #    nn_ii_quer_periodical_mean = nn_ii_quer [np.arange(0,N_one_period) * N_number_periods_second] 
    #    nn_ei_quer_periodical_mean = nn_ei_quer [np.arange(0,N_one_period) * N_number_periods_second]
    #    
    #    c_ee_roh = nn_ee_quer_periodical_mean
    #    c_ii_roh = nn_ii_quer_periodical_mean
    #    c_ei_roh = nn_ei_quer_periodical_mean
    #    
    #    c_ee_naiv = 0.
    #    c_ii_naiv = 0.
    #    c_ei_naiv = 0.    
    #    
    #    freq =  np.fft.fftfreq(N_one_period, d=self.dt_samp)*1000
    #    
    #    return freq, c_ee_roh, c_ii_roh, c_ei_roh, c_ee_naiv, c_ii_naiv, c_ei_naiv