
import nest
import numpy


from parameters_ginzburg_asymm_LFP_new_noisy_mult_theta import *

def simulate(datapath, suffix_general, T, omega = omega_default, h0 = h0_default, masterseed = msd):
    
    suffix = '_h0=' + str(h0) + '_omega=' + str(omega) +  '_T=' + str(T) + suffix_general
    
    print 'Jetzt kommt das suffix'
    print suffix
    print 'Das war das Suffix.'
        
    nest.ResetKernel()
    #nest.SetKernelStatus({"resolution": h, "print_time": False, "overwrite_files": True, 'total_num_virtual_procs': n_virtual_procs, "grng_seed": msd+n_virtual_procs, "rng_seeds" : range(msd+n_virtual_procs+1, msd+2*n_virtual_procs+1)})
    nest.SetKernelStatus({"resolution": h, "print_time": False, "overwrite_files": True, 'total_num_virtual_procs': n_virtual_procs, "grng_seed": masterseed+n_virtual_procs, "rng_seeds" : range(masterseed+n_virtual_procs+1, masterseed+2*n_virtual_procs+1)})

    # this line is crucial to prevent multapses from being created
    # otherwise the decoding of up-/down transitions will go wrong
    nest.sr("/RandomConvergentConnect << /allow_multapses false >> SetOptions")


    nest.SetDefaults("multimeter",
                    {"withtime": False,
                     "withgid": False,
                     "to_memory": False,
                     "to_file": True,
                     "record_from": ["h"],
                     "interval": dt_samp})

    # create recorders for activity
    multi_exc1 = nest.Create("multimeter")
    multi_exc2 = nest.Create("multimeter")
    multi_inh1 = nest.Create("multimeter")
    multi_inh2 = nest.Create("multimeter")
    multi_ext = nest.Create("multimeter")
    
    #Create a sinusodial modulation that perturbs the system:    
    perturbation = nest.Create("ac_generator" , 1 ,{"amplitude": h0 , "frequency": omega/(2*numpy.pi)})
    noise_E      = nest.Create("noise_generator", 1, {"mean": 0., "std": sigma_noise_E, "dt": h, "std_mod": 0., "phase": 0., "frequency": 0.})
    noise_I      = nest.Create("noise_generator", 1, {"mean": 0., "std": sigma_noise_I, "dt": h, "std_mod": 0., "phase": 0., "frequency": 0.})
    
    print "Building network"



    nest.SetDefaults('ginzburg_neuron', {'theta': 0.0, 'tau_m': tau_m, 'c_1': 0.0, 'c_2': 2.*m_x, 'c_3': 1.0})

    # external neurons
    nrnX = nest.Create('ginzburg_neuron', N_X)


    #nest.SetDefaults('mcculloch_pitts_neuron', {'theta': theta[0], 'tau_m': tau_m})
    nest.SetDefaults('mcculloch_pitts_neuron', {'theta': 0.0, 'tau_m': tau_m})
    # excitatory neurons
    nrnE = nest.Create('mcculloch_pitts_neuron', N_E, {'theta': theta[0]})


    #nest.SetDefaults('mcculloch_pitts_neuron', {'theta': theta[1], 'tau_m': tau_m})
    # inhibitory neurons
    nrnI = nest.Create('mcculloch_pitts_neuron', N_I, {'theta': theta[1]})

    # these neurons are only used to record the activity of subpopulations in the network
    # Therefore, the thresholds of these neurons should be irrelevant, right?
    nrn_pre_sum_exc1 = nest.Create('mcculloch_pitts_neuron')
    nrn_pre_sum_exc2 = nest.Create('mcculloch_pitts_neuron')

    nrn_pre_sum_inh1 = nest.Create('mcculloch_pitts_neuron')
    nrn_pre_sum_inh2 = nest.Create('mcculloch_pitts_neuron')

    nrn_pre_sum_ext = nest.Create('mcculloch_pitts_neuron')


    if record_spin:
        # for recording the activity of some single neurons in order to determine 
        # average auto-correlations
        sd_e = nest.Create('spin_detector')
        sd_i = nest.Create('spin_detector')

    nest.SetStatus(multi_exc1, {"label": datapath + "ginzburg-exc1" + suffix })
    #nest.SetStatus(multi_exc1, {"label": datapath + "ginzburg-exc1" + str(omega) + suffix })
    nest.SetStatus(multi_exc2, {"label": datapath + "ginzburg-exc2" + suffix })
    #nest.SetStatus(multi_exc2, {"label": datapath + "ginzburg-exc2" + str(omega)  + suffix })
    nest.SetStatus(multi_inh1, {"label": datapath + "ginzburg-inh1" + suffix })
    #nest.SetStatus(multi_inh1, {"label": datapath + "ginzburg-inh1" + str(omega)  + suffix })
    nest.SetStatus(multi_inh2, {"label": datapath + "ginzburg-inh2" + suffix })
    #nest.SetStatus(multi_inh2, {"label": datapath + "ginzburg-inh2" + str(omega) + suffix })
    if record_spin:
        nest.SetStatus(sd_e, {"to_file": True, "label": datapath + "ginzburg-sd_e" + suffix })
        #nest.SetStatus(sd_e, {"to_file": True, "label": datapath + "ginzburg-sd_e" + str(omega) + suffix })
        nest.SetStatus(sd_i, {"to_file": True, "label": datapath + "ginzburg-sd_i" + suffix })
        #nest.SetStatus(sd_i, {"to_file": True, "label": datapath + "ginzburg-sd_i" + str(omega) + suffix })
        
    print "Creating recurrent connectivity"

    numpy.random.seed(1)

    # recurrent connectivity from excitatory to excitatory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_EE, 'delay': d, 'receptor_type': 0})
    syn_dict={'model': 'static_synapse', 'weight': j_EE, 'delay': d, 'receptor_type': 0}
    conn_dict = {'rule': 'fixed_indegree', 'indegree': K_EE, 'multapses': False}
    nest.Connect(nrnE, nrnE, conn_dict, syn_dict)

    # recurrent connectivity from inhibitory to excitatory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_EI, 'delay': d, 'receptor_type': 0})
    syn_dict={'model': 'static_synapse', 'weight': j_EI, 'delay': d, 'receptor_type': 0}
    conn_dict = {'rule': 'fixed_indegree', 'indegree': K_EI, 'multapses': False}
    nest.Connect(nrnI, nrnE, conn_dict, syn_dict)
    
    # recurrent connectivity from excitatory to inhibitory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_IE, 'delay': d, 'receptor_type': 0})
    syn_dict = {'model': 'static_synapse', 'weight': j_IE, 'delay': d, 'receptor_type': 0}    
    conn_dict ={'rule': 'fixed_indegree', 'indegree': K_IE, 'multapses': False } 
    nest.Connect(nrnE, nrnI, conn_dict, syn_dict)

    # recurrent connectivity from inhibitry to inhibitory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_II, 'delay': d, 'receptor_type': 0})
    syn_dict = {'model': 'static_synapse', 'weight': j_II, 'delay': d, 'receptor_type': 0}
    conn_dict = {'rule': 'fixed_indegree', 'indegree': K_II, 'multapses': False}
    nest.Connect(nrnI, nrnI, conn_dict, syn_dict)


    # connectivity from external to excitatory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_EX, 'delay': d, 'receptor_type': 0})
    syn_dict = {'model': 'static_synapse', 'weight': j_EX, 'delay': d, 'receptor_type': 0}
    conn_dict = {'rule': 'fixed_indegree', 'indegree': K_EX, 'multapses': False}
    nest.Connect(nrnX, nrnE, conn_dict, syn_dict)

    # connectivity from external to inhibitory neurons
    #nest.SetDefaults('static_synapse', {'weight': j_IX, 'delay': d, 'receptor_type': 0})
    syn_dict = {'model': 'static_synapse', 'weight': j_IX, 'delay': d, 'receptor_type': 0}
    conn_dict = {'rule': 'fixed_indegree', 'indegree': K_IX, 'multapses': False}
    nest.Connect(nrnX, nrnI, conn_dict, syn_dict)


    print "Connecting devices."

     #nest.SetDefaults('static_synapse', {'weight': 1.0, 'delay': h})
    syn_dict = {'model': 'static_synapse', 'weight': 1.0, 'delay': h}
    nest.Connect(nrnE[:N_rec_E], nrn_pre_sum_exc1, 'all_to_all', syn_dict)
    nest.Connect(nrnE[N_E/2:N_E/2+N_rec_E], nrn_pre_sum_exc2,'all_to_all', syn_dict)

    nest.Connect(nrnI[:N_rec_I], nrn_pre_sum_inh1, 'all_to_all', syn_dict)
    nest.Connect(nrnI[N_I/2:N_I/2+N_rec_I], nrn_pre_sum_inh2, 'all_to_all', syn_dict)

    nest.Connect(nrnX, nrn_pre_sum_ext,'all_to_all', syn_dict)

    nest.Connect(multi_exc1, nrn_pre_sum_exc1, syn_spec=syn_dict)
    nest.Connect(multi_exc2, nrn_pre_sum_exc2, syn_spec=syn_dict)
    nest.Connect(multi_inh1, nrn_pre_sum_inh1, syn_spec=syn_dict)
    nest.Connect(multi_inh2, nrn_pre_sum_inh2, syn_spec=syn_dict)
    
    #Connect the perturbation to the inhibitory and excitatory neurons
    nest.Connect(perturbation,nrnE,'all_to_all', syn_dict)
    nest.Connect(perturbation,nrnI,'all_to_all', syn_dict)
    
    #Connect the noise to all neurons:
    nest.Connect(noise_I,nrnI,'all_to_all', syn_dict)
    nest.Connect(noise_E,nrnE,'all_to_all', syn_dict)
    
    if input_type == 'binary':
        nest.Connect(multi_ext, nrn_pre_sum_ext, syn_spec=syn_dict)

    if record_spin:
        nest.Connect(nrnE[:n_spin], sd_e,'all_to_all', syn_dict)
        nest.Connect(nrnI[:n_spin], sd_i,'all_to_all', syn_dict)

    nest.Simulate(T)

#datapath = '../data_LFP_new/'
#T=30000.

#T_ext_approx = 2.*np.pi/(omega_default) * 1000.
#print 'T_ext_approx=', T_ext_approx
#n_per_period = int(T_ext_approx/dt_samp)
#T_ext        = n_per_period * dt_samp
#print 'T_ext=', T_ext
#omega        = 1000. * 2* np.pi/T_ext
#print omega 

#simulate(datapath,time_UTC,T, omega)
