import numpy as np
import numpy.linalg as la
import scipy.special as sps
from scipy.optimize import fsolve

from parameters_ginzburg_single_pop import omega_default
from parameters_ginzburg_single_pop import h0_default
from parameters_ginzburg_single_pop import sigma_noise_I
# index convention

I = 0
X = 1

II = 0
IX = 1


class ginzburg:


    def __init__(self, N, J, K, m_x, theta, sigma_noise = np.array([sigma_noise_I]), omega = omega_default, h0 = h0_default):
        self.N = N              #Probably N[E] is the number of excitatory, N[I] the number of inhibitory and N[X] the number of other neurons. Or something...
        self.J = J
        self.K = K
        self.m_x = m_x
        self.theta = theta
        self.omega = omega
        self.h0    = h0
        self.omega_kHz = omega/1000.    #nest measures the frequency in Hz, not in kHz!
        self.sigma_noise = sigma_noise

    def mu_sigma_from_m_corr(self, m, c):
        """calculate the mean and variance of the input to a neuron
        given the mean activity and covariances
        mu_x: mean of noise input
        sigma_x: std of noise input"""
        #print 'Vor Aufruf von np.hstack: m=', m
        #print 'm=', m        
        m_all = np.hstack((m, self.m_x))
        #print 'Nach Aufruf von np.hstack: m=', m        
        #print 'm_all=', m_all        
        C = np.array([ [ c[II], c[IX] ],
                       [ c[IX], 0.    ] ])

        # vector of variances
        a = m_all * (1. - m_all)
                                    # K and J are 3x3-matrices
        KJ  = self.K * self.J          # * multiplies vectors and matrices by components
        KJ2 = self.K * self.J * self.J

        #print 'KJ=', KJ
        #print 'm_all=' , m_all        
        
        mu = np.dot(KJ,m_all)           #Matrix product (formula 4.3.4)
        #print 'Hat doch geklappt!'
        # take into account correlations in the input to determine variance
        sigma2_m = np.dot(KJ2, a)                               # Formula (4.3.7)
        sigma2_c = np.diag( np.dot( np.dot(KJ, C), KJ.T ))      # don't neglect correlations (different to the script)
        
        for ii in range(0,len(sigma2_m)):
            if(sigma2_m[ii]+ sigma2_c[ii] <0):
                print 'sigma2_m[', ii, ']=', sigma2_m[ii]
                print 'sigma2_c[', ii, ']=', sigma2_c[ii]

        
        sigma2 = np.sqrt(sigma2_m + sigma2_c + self.sigma_noise**2)

        return mu, sigma2


#    def susceptibility_corr(self, m, c):
#
#        #print 'susceptibility_corr ruft mu_sigma_from_m_corr auf!'
#        mu, sigma = self.mu_sigma_from_m_corr(m, c)
#        
#        #print 'sigma=', sigma
#        #print 'mu='   , mu        
#        S = 1./(np.sqrt(2.*np.pi)*sigma) * np.exp( -(mu-self.theta)**2 / (2.*sigma**2) ) #Formula (5.3.2)
#	# assumes a (near-)Heaviside gain function
#        return S
    def susceptibility_corr(self, m, c, h = 0.):

        #print 'susceptibility_corr ruft mu_sigma_from_m_corr auf!'
        mu, sigma = self.mu_sigma_from_m_corr(m, c)
        mu = mu + h
        
        #print 'sigma=', sigma
        #print 'mu='   , mu        
        S = 1./(np.sqrt(2.*np.pi)*sigma) * np.exp( -(mu-self.theta)**2 / (2.*sigma**2) ) #Formula (5.3.2)
	# assumes a (near-)Heaviside gain function
        return S


    def correlations(self, m, S):
        """solve the linear system of equations determining the correlations
        given we know the susceptibility"""
        W = ((self.J*self.K).T * S).T           #blubb.T means transpose(blubb) !!!

        N = self.N

        a = m*(1.-m)
        c = np.zeros(2, dtype=float)
#        print 'a[X]=', a[X]
#        print 'self.N[X]=', self.N[X]
#        print 'W=', W        
        
        
        pref  = a[X]/self.N[X] / (2.-W[I,I])   # X=2 (is ne Konstante) cf. notes to Moritz_2014 
        c[IX] = pref * W[I,X]

        M = np.matrix([[1. - W[I,I]]])

        b = np.array([ W[I,I]*a[I]/N[I]])
        #print 'b=', b
        b1 = np.array( [W[I,X]*c[IX]] )
        #print 'b1=', b1
        c[:IX] = np.linalg.solve(M, b + b1)     #Solves (5.6.1) for C
        #print 'c=', c
        return c


#    def solve_self_consistently_corr(self, m0):
#        """determine self-consistent rates and correlations
#        taking effect of correlations on input statistics into account"""
#
#        c = np.zeros(2)
#        
#        def m_zero_corr(m):
#            #print 'solve_self_consistently_corr ruft mu_sigma_from_m_corr auf!'
#            #print 'Vor Aufruf von mu_sigma_from_m_corr: m=', m
#            mu, sigma = self.mu_sigma_from_m_corr(m, c)
#            #print 'Nach Aufruf von mu_sigma_from_m_corr: m=', m
#            new_m = 0.5*(1-sps.erf(-(mu-self.theta)/(np.sqrt(2.)*sigma)))
#
#            return new_m - m
#            
#        err = 1.
#        while err > 1e-15:          #Solves (4.3.8) with the derivative set to zero        
#            m0 = fsolve(m_zero_corr, m0)
#            #print 'Check fuer fsolve (Sollte 0 sein):', m_zero_corr(m0)
#            
#            S = self.susceptibility_corr(m0, c)
#            #print 'S=', S            
#            c_new = self.correlations(np.hstack((m0, self.m_x)), S)
#            #print 'c_new=', c_new            
#            err = la.norm(c_new - c)
#            c = c_new
#
#        #print 'error ', err
#        return m0, c_new
    def solve_self_consistently_corr(self, m0, h=0.):
        """determine self-consistent rates and correlations
        taking effect of correlations on input statistics into account"""

        c = np.zeros(2)
        
        def m_zero_corr(m):
            #print 'solve_self_consistently_corr ruft mu_sigma_from_m_corr auf!'
            #print 'Vor Aufruf von mu_sigma_from_m_corr: m=', m
            
            mu, sigma = self.mu_sigma_from_m_corr(m, c)
            mu = mu + h
            #print 'Nach Aufruf von mu_sigma_from_m_corr: m=', m
            new_m = 0.5*(1-sps.erf(-(mu-self.theta)/(np.sqrt(2.)*sigma)))

            return new_m - m
            
        err = 1.
        while err > 1e-15:          #Solves (4.3.8) with the derivative set to zero        
            m0 = fsolve(m_zero_corr, m0)
            #print 'Check fuer fsolve (Sollte 0 sein):', m_zero_corr(m0)
            
            S = self.susceptibility_corr(m0, c, h)
            #print 'S=', S            
            c_new = self.correlations(np.hstack((m0, self.m_x)), S)
            #print 'c_new=', c_new            
            err = la.norm(c_new - c)
            c = c_new
#        mu, sigma = self.mu_sigma_from_m_corr(m0, c)
        return m0, c_new 
     
    def mu_time_dependent_from_linearization(self, m0, tau, t):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t"""
        # effective connectivity matrix
        N_m=np.size(m0)        
        W = np.zeros((2,2), dtype=float)
        # multiply eah row in effective linearized connectivity by corresponding susceptibility S
        m0, c = self.solve_self_consistently_corr(m0)
        
        S     = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
        
        
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1,:]   = np.zeros(3)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        S_vec      = np.array([S[0],0.])
        #print 'Susceptibilities', S[0], ' und ' , S[1]        
        UinvS_vec  = np.dot(U_inv,S_vec)                 #Direction of the perturbation in the eigen system of the ODE
    
        #print 'Eigenwerte von W:', lambdas
        #print 'S hat folgende Form:', S        
        
        #eps_tilde=np.zeros(N_m+1)
        eps_tilde_list=[]        
        for k in range(0,N_m+1):
            if not (lambdas[k]-1)==0:
                K_k = self.h0 * UinvS_vec[k] * (lambdas[k]-1)/((tau*self.omega_kHz)**2+(lambdas[k]-1)**2)
                eps_tilde_list.append(-K_k*(np.sin(self.omega_kHz *t) + (self.omega_kHz * tau)/(lambdas[k]-1) * np.cos(self.omega_kHz * t)) + K_k * (self.omega_kHz * tau)/(lambdas[k]-1) * np.exp((lambdas[k]-1)*t/tau))
            else:
                eps_tilde_list.append(self.h0 * UinvS_vec[k] * (1./self.omega_kHz*tau)*(1-np.cos(self.omega_kHz*t)))
        #print 'Dimension von W:', np.size(W), 'N_m=', N_m
        eps_tilde = np.array(eps_tilde_list)
        eps       = np.dot(U,eps_tilde)       
        
        return eps
        
    def mu_frequency_dependent_from_linearization(self, m0, tau):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t"""
        # effective connectivity matrix
        N_m = np.size(m0)        
        W   = np.zeros((2,2), dtype=float)
        # multiply eah row in effective linearized connectivity by corresponding susceptibility S
        m0, c = self.solve_self_consistently_corr(m0)
        
        S = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
        
        
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1,:]   = np.zeros(3)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        S_vec      = np.array([S[0],0.])
        #print 'Susceptibilities', S[0], ' und ' , S[1]        
        UinvS_vec=np.dot(U_inv,S_vec)                 #Direction of the perturbation in the eigen system of the ODE
        #print 'UinvS_vec[2]=', UinvS_vec[2]
        #eps_tilde=np.zeros(N_m+1)
        A_k_list = np.zeros(2, dtype=complex)        
        for k in range(0,N_m+1):
            A_k_list[k] = self.h0 * UinvS_vec[k] * (-1j * tau * self.omega_kHz +1 - lambdas[k] ) /((tau*self.omega_kHz)**2+(lambdas[k]-1)**2)
            
        A_final=np.dot(U,A_k_list)       
        #print 'A_final=', A_final
        return A_final
    
    def mu_frequency_dependent_from_linearization_delsig(self, m0, tau):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t"""
        N_m=np.size(m0)        
        W = np.zeros((2,2), dtype=float)
        # multiply eah row in effective linearized connectivity by corresponding susceptibility S
        m0, c = self.solve_self_consistently_corr(m0)
        mu0, sigma0 = self.mu_sigma_from_m_corr(m0, c)

        #print 'm0 und c in mu_frequency_depdendent_from_linearization_delsig:', m0, ' ', c        
        
        S = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
        
        
        W[0:1,:] = ((self.J*self.K).T * S).T  #+ np.dot(np.dot(np.diag(S*(self.theta - mu0)/(2*sigma0**2)),self.K * self.J *self.J),np.identity(len(m0)+1)-2.*np.diag(np.hstack((m0,self.m_x))))    
        W[1,:]   = np.zeros(2)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        S_vec      = np.array([S[0],0.])
        #print 'Susceptibilities', S[0], ' und ' , S[1]        
        UinvS_vec  = np.dot(U_inv,S_vec)                 #Direction of the perturbation in the eigen system of the ODE

        #eps_tilde=np.zeros(N_m+1)
        A_k_list=np.zeros(2, dtype=complex)        
        for k in range(0,N_m+1):
            A_k_list[k] = self.h0 * UinvS_vec[k] * (-1j * tau * self.omega_kHz +1 - lambdas[k] ) /((tau*self.omega_kHz)**2+(lambdas[k]-1)**2)
            
        A_final = np.dot(U,A_k_list)
        
        #print 'W delta m=', np.dot(W,A_final), 'S h_0=', S_vec * self.h0, 'delta m=', A_final        
        #print 'lambdas=', lambdas
        #print 'A_final=', A_final
        return A_final, A_k_list, lambdas    
    
    def inhomogeneity_cquer_from_mu_time_dependent(self, kk, ii, jj, m0, tau,t):
        """Calculates for c_{i,j} (where c is the correlation matrix) the solution of the ODE with 
        the inhomogeneity arising from the deviation eps_k of the mean m_k from the stationary solution"""
        # multiply eah row in effective linearized connectivity by corresponding susceptibility S
            
        m0, c = self.solve_self_consistently_corr(m0)
        
        S = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
        # effective connectivity matrix
        W = np.zeros((2,2), dtype=float)
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        S_vec      = np.array([S[0],0.])
        #print 'Susceptibilities', S[0], ' und ' , S[1]        
        UinvS_vec=np.dot(U_inv,S_vec)                 #Direction of the perturbation in the eigen system of the ODE
                                    
        #eps_tilde=np.zeros(N_m+1)
        
        part_sin       = 0.
        part_cos       = 0.
        part_exp_inhom = 0.               #The part of the inhomogeneous solution decaying exponentially with -tau/(lambdas[k]-1) or \propto t exp(t*(lambdas[ii]+lambdas[jj]-2)/tau)
        part_exp_hom   = 0.                 #The homogeneous part decaying exponentially with -tau/(lambdas[ii]+lambdas[jj]-1)
                
        K_k = - self.h0 * UinvS_vec[kk] * (lambdas[kk]-1)/((tau*self.omega_kHz)**2+(lambdas[kk]-1)**2)
        C_k = - self.h0 * UinvS_vec[kk] /(self.omega_kHz * tau)**2      #C_k := lim_lambda->1 {K_k/lambda-1}
        K_ij=(self.omega_kHz*tau)/((self.omega_kHz*tau)**2+(lambdas[ii]+lambdas[jj]-2)**2)                   
        if not (lambdas[kk]-1)==0:
            part_sin = K_k * K_ij * (-(lambdas[ii]+lambdas[jj]-2)/(self.omega_kHz*tau)+(self.omega_kHz*tau/(lambdas[kk]-1)))
            part_cos = K_k * K_ij * (-1.-(lambdas[ii]+lambdas[jj]-2.)/(lambdas[kk]-1.))
            if not (lambdas[kk]==lambdas[ii]+lambdas[jj]-1.):
                part_exp_inhom = K_k * (-(self.omega_kHz*tau)/((lambdas[kk]-1)*(lambdas[kk]-lambdas[ii]-lambdas[jj]+1.)))
                part_exp_hom   = -part_cos - part_exp_inhom
            else:        #If lambdas[kk]-1 == lambdas[ii]+lambdas[jj]-2, we cannot differentiate between the decay rate of the homogeneour ODE and the decay rate of the inhomogeneity
                        #We chose to assign the prefactor to the homogeneous part
                part_exp_inhom = 0.
                part_exp_hom   = -part_cos + K_k * ( - t * self.omega_kHz/(lambdas[kk]-1))
                print 'Ausnahme eingetreten. lambdas[kk]=lambdas[ii]+lambdas[jj]-1'
        else:
            print 'Ausnahme eingetreten. lambdas[kk]==1'            
            part_sin = C_k * K_ij * self.omega_kHz * tau           
            part_cos = C_k * K_ij * ( -(lambdas[ii]+lambdas[jj]-2.)) 
            if not(lambdas[ii]+lambdas[jj]-2==0):
                part_exp_inhom = -self.omega_kHz*tau/(-lambdas[ii]-lambdas[jj]+2)
                part_exp_hom   = -part_cos - part_exp_inhom
            else:
                part_exp_inhom = 0.
                part_exp_hom   = -self.omega_kHz * t - part_cos
        
        
        return part_sin, part_cos, part_exp_inhom, part_exp_hom    

    def inhomogeneity_c_from_mu_time_dependent(self, kk, ii, m0, tau,t):
        """Calculates for c_{i,j} (where c is the correlation matrix) the solution of the ODE with 
        the inhomogeneity arising from the deviation eps_k of the mean m_k from the stationary solution"""
        # multiply eah row in effective linearized connectivity by corresponding susceptibility S
            
        m0, c = self.solve_self_consistently_corr(m0)
        
        S = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
        # effective connectivity matrix
        W = np.zeros((3,3), dtype=float)
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :] = np.zeros(3)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        S_vec      = np.array([S[0],0.])
        #print 'Susceptibilities', S[0], ' und ' , S[1]        
        UinvS_vec=np.dot(U_inv,S_vec)                 #Direction of the perturbation in the eigen system of the ODE
         
        #eps_tilde=np.zeros(N_m+1)
        
        part_sin=0.
        part_cos=0.
        part_exp_inhom=0.               #The part of the inhomogeneous solution decaying exponentially with -tau/(lambdas[k]-1) or \propto t exp(t*(lambdas[ii]-1)/tau)
        part_exp_hom=0.                 #The homogeneous part decaying exponentially with -tau/(lambdas[ii]-1)
        
        K_k = - self.h0 * UinvS_vec[kk] * (lambdas[kk]-1)/((tau*self.omega_kHz)**2+(lambdas[kk]-1)**2)
        C_k = - self.h0 * UinvS_vec[kk] /(self.omega_kHz * tau)**2      #C_k := lim_lambda->1 {K_k/lambda-1}
        K_ij=(self.omega_kHz*tau)/((self.omega_kHz*tau)**2+(lambdas[ii]-1.)**2)                   
        if not (lambdas[kk]-1)==0:
            part_sin = K_k * K_ij * (-(lambdas[ii]-1.)/(self.omega_kHz*tau)+(self.omega_kHz*tau/(lambdas[kk]-1)))
            part_cos = K_k * K_ij * (-1.-(lambdas[ii]-1.)/(lambdas[kk]-1.))
            if not (lambdas[kk]==lambdas[ii]):
                part_exp_inhom = K_k * (-(self.omega_kHz*tau)/((lambdas[kk]-1)*(lambdas[kk]-lambdas[ii])))
                part_exp_hom   = -part_cos - part_exp_inhom
            else:        #If lambdas[kk]-1 == lambdas[ii]-1, we cannot differentiate between the decay rate of the homogeneour ODE and the decay rate of the inhomogeneity
                        #We chose to assign the prefactor to the homogeneous part
                part_exp_inhom = 0.
                part_exp_hom   = -part_cos + K_k * ( - t * self.omega_kHz/(lambdas[kk]-1))
                print 'Ausnahme eingetreten. lambdas[kk]==lambdas[ii]'
        else:
            print 'Ausnahme eingetreten. lambdas[kk]==1'            
            part_sin = C_k * K_ij * self.omega_kHz * tau           
            part_cos = C_k * K_ij * ( -(lambdas[ii]-1.)) 
            if not(lambdas[ii]-1==0):
                part_exp_inhom = -self.omega_kHz*tau/(-lambdas[ii]+1.)
                part_exp_hom   = -part_cos - part_exp_inhom
            else:
                part_exp_inhom = 0.
                part_exp_hom   = -self.omega_kHz * t - part_cos
        
        
        return part_sin, part_cos, part_exp_inhom, part_exp_hom    
    
    def delta_cquer_same_time_from_linearization(self, N_pop_size, m0_no_x, tau, t):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m = np.size(m0_no_x)+1
        N_t = np.size(t)
        #eps=self.mu_time_dependent_from_linearization(m0,tau, t)
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        
        a0 = m0*(1.-m0)
        #print 'delta_cquer_same_time_from_linearization ruft mu_sigma_from_m_corr auf!'
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
              
        Z= np.zeros((2,2), dtype=float)        
        V= np.zeros((2,2), dtype=float)        
        W= np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K
        Z[1,:]   = np.zeros(2)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(2)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        m0_diag=np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        a0_diag=np.diag(a0/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps))        
        
        #print 'V=', V, 'a0_diag=', a0_diag, 'c0=', c0        
        
        Vc_plus_a=np.dot(V,a0_diag + c0)
        
        Inhom_mu_solution_eigensys = np.zeros((N_t,N_m), dtype=float)
        Inhom_mu_solution = np.zeros((N_t,N_m), dtype=float)
        
        Inhom_from_eps_eigensys=np.zeros((N_t,N_m,N_m), dtype=float)
        Inhom_from_eps=np.zeros((N_t,N_m,N_m), dtype=float)        
   
        #Calculate the solution for the ODE with the inhomogeneity stemming from eps
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                for epseps in range(0,N_m):            
                    part_sin, part_cos, part_exp_inhom, part_exp_hom  =  self.inhomogeneity_cquer_from_mu_time_dependent(epseps, ii, jj, m0[0:2], tau,t)
                    #print 'part_sin, part_cos, part_exp_inhom, part_exp_hom=', part_sin, part_cos, part_exp_inhom, part_exp_hom
                    #print 'part_cos + part_exp_inhom + part_exp_hom', part_cos + part_exp_inhom + part_exp_hom                    
                    Inhom_mu_solution_eigensys[:,epseps] =  part_cos * np.cos(self.omega_kHz*t) + part_sin * np.sin(self.omega_kHz*t) + part_exp_inhom * np.exp((lambdas[epseps]-1)*t/tau) + part_exp_hom * np.exp((lambdas[ii]+lambdas[jj]-2.)*t/tau)  
                    #print 'Inhom_mu_solution_eigensys[epseps]=', Inhom_mu_solution_eigensys[epseps]
                Inhom_mu_solution=np.dot(U,Inhom_mu_solution_eigensys.T).T          #In the i'th row, there are the values of the i'th time value. 
                                                                                    #Therefore, transponation, matrix multiplication and transponation againg does the right thing
                KJ_imhom_from_eps_mat=np.zeros((N_t,N_m,N_m), dtype=float)                
                for i_t in range(0,N_t):                
                    KJ_imhom_from_eps_mat[i_t,:,:] = np.diag(np.dot(Z,Inhom_mu_solution[i_t,:]))
                    Inhom_from_eps_eigensys[i_t,ii,jj]=np.dot(np.dot(U_inv[ii,:],np.dot(KJ_imhom_from_eps_mat[i_t,:,:],Vc_plus_a)+np.dot(np.diag(Inhom_mu_solution[i_t,:]),np.dot(W,(1-2.*m0_diag)/N_pop_size))),U_inv.T[:,jj])
                    Inhom_from_eps_eigensys[i_t,:,:]=Inhom_from_eps_eigensys[i_t,:,:]+Inhom_from_eps_eigensys[i_t,:,:].T
        
        for i_t in range(0,N_t):
            Inhom_from_eps[i_t,:,:]=np.dot(U,np.dot(Inhom_from_eps_eigensys[i_t,:,:],U.T))        
        #Calculate the solution for the ODE with the inhomogeneity stemming from h0 sin(self.omega_kHz t)
        
        Inhom_from_sin_eigensys=np.zeros((N_t,N_m,N_m), dtype=float)
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                time_indep                        =  np.dot(U_inv[ii,:],np.dot(Vc_plus_a,U_inv.T[:,jj])) * (-1/self.omega_kHz) *(self.omega_kHz*tau)**2/((self.omega_kHz*tau)**2+(lambdas[ii]+lambdas[jj]-2.)**2)
                time_dep                          =  self.h0*((np.cos(self.omega_kHz*t)-np.exp(t*(lambdas[ii]+lambdas[jj]-2)/tau)+((lambdas[ii]+lambdas[jj]-2.)/(self.omega_kHz*tau))*np.sin(self.omega_kHz*t)))
                Inhom_from_sin_eigensys[:,ii,jj]  =  time_indep * time_dep
        
        Inhom_from_sin=np.zeros((N_t,N_m,N_m), dtype=float)
        for i_t in range(0,N_t):        
            Inhom_from_sin_eigensys[i_t,:,:]=Inhom_from_sin_eigensys[i_t,:,:]+Inhom_from_sin_eigensys[i_t,:,:].T
            Inhom_from_sin[i_t,:,:]=np.dot(U,np.dot(Inhom_from_sin_eigensys[i_t,:,:],U.T))
        
        return Inhom_from_eps+Inhom_from_sin
        
        
#==============================================================================
        ######################################################
#         Bis hierhin einmal Korrektur gelesen
        ######################################################
#==============================================================================

    def delta_cquer_same_time_from_linearization_frequencyspace(self, N_pop_size, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m=np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        
        a0=m0*(1.-m0)
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
              
        Z= np.zeros((2,2), dtype=float)        
        V= np.zeros((2,2), dtype=float)        
        W= np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K
        Z[1,:]   = np.zeros(2)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(2)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        m0_diag = np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        a0_diag = np.diag(a0/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps))        
        
        #print 'V=', V, 'a0_diag=', a0_diag, 'c0=', c0        
        
        Vc_plus_a = np.dot(V,a0_diag + c0)
        
        Inhom_mu_solution_eigensys = np.zeros(N_m, dtype=float)
        Inhom_mu_solution          = np.zeros(N_m, dtype=float)
        
        Inhom_from_eps_eigensys = np.zeros((N_m,N_m), dtype=float)
        Inhom_from_eps          = np.zeros((N_m,N_m), dtype=float)        
   
        #Calculate the solution for the ODE with the inhomogeneity stemming from eps
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                for epseps in range(0,N_m):            
                    part_sin, part_cos, part_exp_inhom, part_exp_hom  =  self.inhomogeneity_cquer_from_mu_time_dependent(epseps, ii, jj, m0[0:2], tau,t)
                    #print 'part_sin, part_cos, part_exp_inhom, part_exp_hom=', part_sin, part_cos, part_exp_inhom, part_exp_hom
                    #print 'part_cos + part_exp_inhom + part_exp_hom', part_cos + part_exp_inhom + part_exp_hom                    
                    Inhom_mu_solution_eigensys[:,epseps] =  part_cos * 1j + part_sin  
                    #print 'Inhom_mu_solution_eigensys[epseps]=', Inhom_mu_solution_eigensys[epseps]
                Inhom_mu_solution=np.dot(U,Inhom_mu_solution_eigensys.T).T          #In the i'th row, there are the values of the i'th time value. 
                                                                                    #Therefore, transponation, matrix multiplication and transponation againg does the right thing
                KJ_imhom_from_eps_mat = np.diag(np.dot(Z,Inhom_mu_solution[:]))
                Inhom_from_eps_eigensys[ii,jj]=np.dot(np.dot(U_inv[ii,:],np.dot(KJ_imhom_from_eps_mat,Vc_plus_a)+np.dot(np.diag(Inhom_mu_solution[:]),np.dot(W,(1-2.*m0_diag)/N_pop_size))),U_inv.T[:,jj])
                Inhom_from_eps_eigensys[i_t,:,:]=Inhom_from_eps_eigensys[i_t,:,:]+Inhom_from_eps_eigensys[i_t,:,:].T
        
        for i_t in range(0,N_t):
            Inhom_from_eps[i_t,:,:]=np.dot(U,np.dot(Inhom_from_eps_eigensys[i_t,:,:],U.T))        
        #Calculate the solution for the ODE with the inhomogeneity stemming from h0 sin(self.omega_kHz t)
        
        Inhom_from_sin_eigensys=np.zeros((N_t,N_m,N_m), dtype=float)
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                time_indep                        =  np.dot(U_inv[ii,:],np.dot(Vc_plus_a,U_inv.T[:,jj])) * (-1/self.omega_kHz) *(self.omega_kHz*tau)**2/((self.omega_kHz*tau)**2+(lambdas[ii]+lambdas[jj]-2.)**2)
                time_dep                          =  self.h0*((np.cos(self.omega_kHz*t)-np.exp(t*(lambdas[ii]+lambdas[jj]-2)/tau)+((lambdas[ii]+lambdas[jj]-2.)/(self.omega_kHz*tau))*np.sin(self.omega_kHz*t)))
                Inhom_from_sin_eigensys[:,ii,jj]  =  time_indep * time_dep
        
        Inhom_from_sin=np.zeros((N_t,N_m,N_m), dtype=float)
        for i_t in range(0,N_t):        
            Inhom_from_sin_eigensys[i_t,:,:]=Inhom_from_sin_eigensys[i_t,:,:]+Inhom_from_sin_eigensys[i_t,:,:].T
            Inhom_from_sin[i_t,:,:]=np.dot(U,np.dot(Inhom_from_sin_eigensys[i_t,:,:],U.T))
        
        return Inhom_from_eps+Inhom_from_sin    
    
    
    def delta_cquer_same_time_from_linearization_frequencyspace_consistencycheck_delsig(self, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m = np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        
        a0 = m0*(1.-m0)
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
        
        A, ente, gans = self.mu_frequency_dependent_from_linearization_delsig(m0_no_x, tau)        
        
        Z = np.zeros((2,2), dtype=float)        
        V = np.zeros((2,2), dtype=float)        
        W = np.zeros((2,2), dtype=float)        
        
        #Z[0:2,:]= self.J*self.K
        
        Z[0:1,:] = self.J*self.K #+np.dot(np.dot(np.diag(((self.theta-mu)/sigma - sigma/(self.theta - mu)) * (1/(2*sigma)) ), self.K*self.J*self.J), np.diag(1.-2.*m0))
        Z[1,:]   = np.zeros(2)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(2)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0
        
        #print 'W=', W
#        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        #if(np.linalg.matrix_rank(W)!=1):
        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        #else:   # Is also always holds W[i,0] = W[i,2] forall i and W[0,j] = W[1,j] forall j
            #print 'rank(W)=1'
#            lambdas    = np.zeros(2, dtype = complex)            
#            lambdas[0] = W[0,0]+W[0,1]
#            U          = np.zeros((3,3), dtype = complex)
#            U[:,0]     = 1./np.sqrt(2.) * np.array([1.,1.,0.])
#            U[:,1]     = 1./np.sqrt(W[0,0]**2 + W[0,1]**2) * np.array([-W[0,1], W[0,0], 0. ])
#            U[:,2]     = 1./np.sqrt(2.) * np.array([1.,0.,-1.])
        #print 'U=', U                
        #print 'lambdas=', lambdas        
        U_inv = np.linalg.inv(U)
        
                
        
        #m0_diag=np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        a0_diag = np.diag(a0/self.N)
        #Ueps_mat = np.diag(np.dot(U,eps))        
        
        #print 'V=', V, 'a0_diag=', a0_diag, 'c0=', c0        
        
        Vc_plus_a = np.dot(V,a0_diag + c0)
        prefactor_delta_c_eigen = np.zeros((N_m,N_m), dtype =complex)        
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                prefactor_delta_c_eigen[ii,jj]  = (1j * self.omega_kHz * tau + 2. - lambdas[ii]-lambdas[jj])
        #print 'prefactor_delta_c_eigen=', prefactor_delta_c_eigen     
        right_side_natural_basis = np.dot(np.diag(np.dot(Z,A)), Vc_plus_a) + np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(A))) + self.h0 * Vc_plus_a
        right_side_natural_basis = right_side_natural_basis + np.transpose(right_side_natural_basis)                      # Make the right side symmetrical
        right_side_eigen         = np.dot(U_inv,np.dot(right_side_natural_basis,np.transpose(U_inv)))
        #print 'right_side_natural_basis=', right_side_natural_basis        
        #print 'right_side_eigen=', right_side_eigen
        #print 'U=', U        
        B_eigen = np.zeros((N_m,N_m), dtype =complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                B_eigen[ii,jj] = right_side_eigen[ii,jj]/prefactor_delta_c_eigen[ii,jj]
        
        B = np.dot(np.dot(U,B_eigen),np.transpose(U))
        
        ################################################
        ### Calculate every contribution seperately ####
        ################################################        
        
        right_side_natural_basis_S_m = np.dot(np.diag(np.dot(Z,A)), Vc_plus_a)
        right_side_natural_basis_S_m = right_side_natural_basis_S_m + np.transpose(right_side_natural_basis_S_m)                      # Make the right side symmetrical
        right_side_eigen_S_m         = np.dot(U_inv,np.dot(right_side_natural_basis_S_m,np.transpose(U_inv)))
        
        right_side_natural_basis_S_h = self.h0 * Vc_plus_a
        right_side_natural_basis_S_h = right_side_natural_basis_S_h + np.transpose(right_side_natural_basis_S_h)                      # Make the right side symmetrical
        right_side_eigen_S_h         = np.dot(U_inv,np.dot(right_side_natural_basis_S_h,np.transpose(U_inv)))
        
        right_side_natural_basis_a_m = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(A)))
        right_side_natural_basis_a_m = right_side_natural_basis_a_m + np.transpose(right_side_natural_basis_a_m)                      # Make the right side symmetrical
        right_side_eigen_a_m         = np.dot(U_inv,np.dot(right_side_natural_basis_a_m,np.transpose(U_inv)))
        
        B_eigen_S_m = np.zeros((N_m,N_m), dtype =complex)
        B_eigen_S_h = np.zeros((N_m,N_m), dtype =complex)      
        B_eigen_a_m = np.zeros((N_m,N_m), dtype =complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                B_eigen_S_m[ii,jj] = right_side_eigen_S_m[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                B_eigen_S_h[ii,jj] = right_side_eigen_S_h[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                B_eigen_a_m[ii,jj] = right_side_eigen_a_m[ii,jj]/prefactor_delta_c_eigen[ii,jj]

        
        B_S_m = np.dot(np.dot(U,B_eigen_S_m),np.transpose(U))
        B_S_h = np.dot(np.dot(U,B_eigen_S_h),np.transpose(U))
        B_a_m = np.dot(np.dot(U,B_eigen_a_m),np.transpose(U))
        return B, B_eigen, B_S_m, B_S_h, B_a_m    #, B_eigen    
        
    def delta_cquer_same_time_from_linearization_frequencyspace_consistencycheck(self, N_pop_size, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m=np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        
        a0=m0*(1.-m0)
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
        
        A = self.mu_frequency_dependent_from_linearization(m0_no_x, tau)        
        
        Z = np.zeros((2,2), dtype=float)        
        V = np.zeros((2,2), dtype=float)        
        W = np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K
        #Z[0:2,:]= self.J*self.K -np.dot(np.dot(np.diag((mu-self.theta)/(2.*sigma**2)), self.K*self.J*self.J), np.diag(1.-2.*m0))
        Z[1,:]  = np.zeros(3)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :] = np.zeros(3)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        #m0_diag=np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        a0_diag = np.diag(a0/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps))        
        
        #print 'V=', V, 'a0_diag=', a0_diag, 'c0=', c0        
        
        Vc_plus_a = np.dot(V,a0_diag + c0)
        prefactor_delta_c_eigen = np.zeros((N_m,N_m), dtype =complex)        
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                prefactor_delta_c_eigen[ii,jj]  = (1j * self.omega_kHz * tau + 2 - lambdas[ii]-lambdas[jj])
        #print 'prefactor_delta_c_eigen=', prefactor_delta_c_eigen     
        right_side_natural_basis = np.dot(np.diag(np.dot(Z,A)), Vc_plus_a) + np.dot(W,np.dot(np.diag(1-2.*m0)/N_pop_size, np.diag(A))) + self.h0 * Vc_plus_a
        right_side_natural_basis = right_side_natural_basis + np.transpose(right_side_natural_basis)                      # Make the right side symmetrical
        right_side_eigen         = np.dot(U_inv,np.dot(right_side_natural_basis,np.transpose(U_inv)))
        B_eigen = np.zeros((N_m,N_m), dtype =complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                B_eigen[ii,jj] = right_side_eigen[ii,jj]/prefactor_delta_c_eigen[ii,jj]
        
        B = np.dot(np.dot(U,B_eigen),np.transpose(U))
        return B
    
    def delta_cquer_bare_same_time_from_linearization_frequencyspace_consistencycheck_delsig(self, N_pop_size, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m=np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
        
        A, ente, gans = self.mu_frequency_dependent_from_linearization_delsig(m0_no_x, tau)        
        
        Z = np.zeros((2,2), dtype=float)        
        V = np.zeros((2,2), dtype=float)        
        W = np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K +np.dot(np.dot(np.diag(((self.theta-mu)/sigma - sigma/(self.theta - mu)) * (1/(2*sigma)) ), self.K*self.J*self.J), np.diag(1.-2.*m0))
        Z[1,:]   = np.zeros(3)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(3)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        m0_diag=np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps)) 
        Vc_plus_m=np.dot(V,m0_diag+ c0)
        prefactor_delta_c_eigen = np.zeros((N_m,N_m), dtype =complex)        
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                prefactor_delta_c_eigen[ii,jj]  = (1j * self.omega_kHz * tau + 2 - lambdas[ii]-lambdas[jj])
        #print 'prefactor_delta_c_eigen=', prefactor_delta_c_eigen     
        right_side_natural_basis = np.dot(np.diag(np.dot(Z,A)), Vc_plus_m) + np.dot(W,np.dot(1./N_pop_size, np.diag(A))) + self.h0 * Vc_plus_m
        right_side_natural_basis = right_side_natural_basis + np.transpose(right_side_natural_basis)                      # Make the right side symmetrical
        right_side_eigen         = np.dot(U_inv,np.dot(right_side_natural_basis,np.transpose(U_inv)))
        B_eigen = np.zeros((N_m,N_m), dtype =complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                B_eigen[ii,jj] = right_side_eigen[ii,jj]/prefactor_delta_c_eigen[ii,jj]
        
        B = np.dot(np.dot(U,B_eigen),np.transpose(U))
        return B  
        
    def delta_cquer_bare_same_time_from_linearization_frequencyspace_consistencycheck(self, N_pop_size, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m=np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
        
        A= self.mu_frequency_dependent_from_linearization(m0_no_x, tau)        
        
        Z = np.zeros((2,2), dtype=float)        
        V = np.zeros((2,2), dtype=float)        
        W = np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K
        Z[1,:]   = np.zeros(3)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(3)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        m0_diag = np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps)) 
        Vc_plus_m = np.dot(V,m0_diag+ c0)
        prefactor_delta_c_eigen = np.zeros((N_m,N_m), dtype =complex)        
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                prefactor_delta_c_eigen[ii,jj]  = (1j * self.omega_kHz * tau + 2 - lambdas[ii]-lambdas[jj])
        #print 'prefactor_delta_c_eigen=', prefactor_delta_c_eigen     
        right_side_natural_basis = np.dot(np.diag(np.dot(Z,A)), Vc_plus_m) + np.dot(W,np.dot(1./N_pop_size, np.diag(A))) + self.h0 * Vc_plus_m
        right_side_natural_basis = right_side_natural_basis + np.transpose(right_side_natural_basis)                      # Make the right side symmetrical
        right_side_eigen         = np.dot(U_inv,np.dot(right_side_natural_basis,np.transpose(U_inv)))
        B_eigen = np.zeros((N_m,N_m), dtype =complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                B_eigen[ii,jj] = right_side_eigen[ii,jj]/prefactor_delta_c_eigen[ii,jj]
        
        B = np.dot(np.dot(U,B_eigen),np.transpose(U))
        return B  
            
    
    def delta_c_from_linearization(self, N_pop_size, m0_no_x, tau, t):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m=np.size(m0_no_x)+1
        N_t=np.size(t)
        #eps=self.mu_time_dependent_from_linearization(m0,tau, t)
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        #print 'm0_no_x=', m0_no_x
        #print 'c0_vec=' , c0_vec        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))

        
        a0 = m0*(1.-m0)
        #print 'delta_cquer_same_time_from_linearization ruft mu_sigma_from_m_corr auf!'
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        c0 = np.array([ [ c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[IX], 0.    ] ])        
              
        Z= np.zeros((2,2), dtype=float)        
        V= np.zeros((2,2), dtype=float)        
        W= np.zeros((2,2), dtype=float)        
        
        Z[0:1,:] = self.J*self.K
        Z[1,:]   = np.zeros(3)        
        
        V[0:1,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T    
        V[1, :]  = np.zeros(3)           
                
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0
        

        lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv      = np.linalg.inv(U)
        
        m0_diag = np.diag(m0)
        #eps_diag=np.diag(eps/N_pop_size)
        a0_diag = np.diag(a0/N_pop_size)
        #Ueps_mat = np.diag(np.dot(U,eps))        
        
        #print 'V=', V, 'a0_diag=', a0_diag, 'c0=', c0        
        
        Vc_plus_a = np.dot(V,a0_diag + c0)
        
        Inhom_mu_solution_eigensys = np.zeros((N_t,N_m), dtype=float)
        Inhom_mu_solution = np.zeros((N_t,N_m), dtype=float)
        
        Inhom_from_eps_eigensys=np.zeros((N_t,N_m,N_m), dtype=float)
        Inhom_from_eps=np.zeros((N_t,N_m,N_m), dtype=float)        
        #print 'N_m=', N_m        
        #Calculate the solution for the ODE with the inhomogeneity stemming from eps
        for ii in range(0,N_m):        
            for epseps in range(0,N_m):            
                part_sin, part_cos, part_exp_inhom, part_exp_hom  =  self.inhomogeneity_c_from_mu_time_dependent(epseps, ii, m0[0:2], self.omega_kHz, tau,t)
                Inhom_mu_solution_eigensys[:,epseps]                =  part_cos * np.cos(self.omega_kHz*t) + part_sin * np.sin(self.omega_kHz*t) + part_exp_inhom * np.exp((lambdas[epseps]-1)*t/tau) + part_exp_hom * np.exp((lambdas[ii]-1.)*t/tau)  
                Inhom_mu_solution=np.dot(U,Inhom_mu_solution_eigensys.T).T          #In the i'th column, there are the values of the i'th time value. Therefore, matrix multiplication does the right thing
#==============================================================================
#                 print 'Inhom_mu_solution=', Inhom_mu_solution
#                 print 'U=', U
#==============================================================================
                KJ_imhom_from_eps_mat=np.zeros((N_t,N_m,N_m), dtype=float)                
                for i_t in range(0,N_t):                
                    KJ_imhom_from_eps_mat[i_t,:,:] = np.diag(np.dot(Z,Inhom_mu_solution[i_t,:]))
                    Inhom_from_eps_eigensys[i_t,ii,:]=np.dot(U_inv[ii,:],np.dot(KJ_imhom_from_eps_mat[i_t,:,:],Vc_plus_a)+np.dot(np.diag(Inhom_mu_solution[i_t,:]),np.dot(W,(1-2.*m0_diag)/N_pop_size)))
                    
        for i_t in range(0,N_t):
            Inhom_from_eps[i_t,:,:]=np.dot(U,Inhom_from_eps_eigensys[i_t,:,:])
        #Calculate the solution for the ODE with the inhomogeneity stemming from h0 sin(self.omega_kHz t)
        
        Inhom_from_sin_eigensys=np.zeros((N_t,N_m,N_m), dtype=float)
        for ii in range(0,N_m):
            time_indep                       =  np.dot(U_inv[ii,:],Vc_plus_a) * (-1/self.omega_kHz) *(self.omega_kHz*tau)**2/((self.omega_kHz*tau)**2+(lambdas[ii]-1.)**2)
            time_dep                         =  self.h0*((np.cos(self.omega_kHz*t)-np.exp(t*(lambdas[ii]-1)/tau)+((lambdas[ii]-1.)/(self.omega_kHz*tau))*np.sin(self.omega_kHz*t)))
            Inhom_from_sin_eigensys[:,ii,:]  =  time_indep * time_dep
        
        Inhom_from_sin=np.zeros((N_t,N_m,N_m), dtype=float)
        
        for i_t in range(0,N_t):           
            Inhom_from_sin[i_t,:,:]=np.dot(U,Inhom_from_sin_eigensys[i_t,:,:])
        
        return Inhom_from_eps+Inhom_from_sin 

    
        
    def corr_vs_lag(self, m0, tau, tlag):
        """determine intrinsic average pairwise cross-correlations as a 
        function of time lag using eigendecomposition of connectivity matrix
        m0: starting guess for mean activities
        tlag should be in the form [max,step] and the result should include 0"""

        # left and right range of time points
        ts_left = np.arange(-tlag[0], 0., tlag[1])     
        ts_right = np.arange(0., tlag[0]+tlag[1], tlag[1])

        # zero-lag correlations required to obtain susceptibility
        m, c0 = self.solve_self_consistently_corr(m0)
        S = self.susceptibility_corr(m, c0)

        m = np.hstack( (m, np.array([self.m_x])) )
        print 'mean activities = ', m

        # effective connectivity matrix
        W = np.zeros((2,2), dtype=float)
        # multiply each row in effective linearized connectivity by corresponding susceptibility S
        W[0:1,:] = ((self.J*self.K).T * S).T    
        W[1, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0

        print 'effective connectivity', W
        
        M = np.diag([1.,1.])-W

        A = np.diag(m*(1.-m)/self.N)

        #print 'A = ', A

        # right eigenvectors
        lmbds, U = np.linalg.eig(M)
        n = len(lmbds)

        # each row of V is one left-sided eigenvector
        V = np.linalg.inv(U)

        # now each row is one right-sided eigenvector
        U = U.T
        
        # check bi-orthogonality
        #print 'should be unit matrix', np.dot(V.T,U)

        Cbar0 = np.zeros([len(lmbds), len(lmbds)], dtype=float)

        C = np.zeros((n, n, len(ts_left)+len(ts_right)), dtype=float)

        for i,lmbd_i in enumerate(lmbds):       #formula (5.6.9)
            for k,lmbd_k in enumerate(lmbds):
                A_ik = np.dot( V[i], np.dot(A, V[k]) ) #V[i] gives the i'th row of V
                Cbar_ik = 2.*A_ik / (lmbd_i + lmbd_k) * np.outer( U[i], U[k] ) 
                Cbar0 += Cbar_ik
                C[:,:,:len(ts_left)] += np.outer ( Cbar_ik.T.flatten(), np.exp(lmbd_k * ts_left/tau) ).reshape((n,n,len(ts_left)))
                C[:,:,len(ts_left):] += np.outer ( Cbar_ik.flatten(), np.exp(-lmbd_k * ts_right/tau) ).reshape((n,n,len(ts_right)))

        C[:,:,:len(ts_left)] -= np.outer ( A.T.flatten(), np.exp(ts_left/tau) ).reshape((n,n,len(ts_left)))     #Substract auto-correlations
        C[:,:,len(ts_left):] -= np.outer ( A.flatten(), np.exp(-ts_right/tau) ).reshape((n,n,len(ts_right)))


        print 'cov matrix (bar)', Cbar0
        print 'cov matrix', Cbar0 - A
        print 'c0', c0
        print 'with time dep', C
        
        return np.hstack( (ts_left, ts_right) ), C