# -*- coding: utf-8 -*-
"""
Created on Wed Dec 17 20:45:57 2014

@author: kuehn
"""

import numpy as np
#import fig_binary_network_scaling_asymm_frequencyspace as fuppes
from ginzburg_theory_single_pop import ginzburg
import matplotlib as mp
from matplotlib.pylab import *
import parameters_ginzburg_single_pop as params

from scipy.integrate import ode
from scipy.special import erf

I = 0
X = 1

II = 0
IX = 1

def phi(mu,sigma):
    return 0.5-0.5*erf((params.theta-mu)/(np.sqrt(2.)*sigma))

def susceptibility_corr(mu, sigma):
        S = 1./(np.sqrt(2.*np.pi)*sigma) * np.exp( -(mu-params.theta)**2 / (2.*sigma**2) ) #Formula (5.3.2), assumes a (near-)Heaviside gain function
        return S

def mu_sigma_from_m_corr(m, c):
    """calculate the mean and variance of the input to a neuron
    given the mean activity and covariances
    mu_x: mean of noise input
    sigma_x: std of noise input"""
    C = np.array([ [ c[II], c[IX] ],
                   [ c[IX], 0.    ] ])    
    
    # vector of variances    
    a = m*(1. - m)

    KJ  = params.K_array*params.J_array                      # * multiplies vectors and matrices by components
    KJ2 = params.K_array*params.J_array*params.J_array

    mu = np.dot(KJ,m)              #Matrix product (formula 4.3.4)
    # take into account correlations in the input to determine variance
    sigma2_m = np.dot(KJ2, a)                               # Formula (4.3.7)
    sigma2_c = np.diag( np.dot( np.dot(KJ, C), KJ.T ))      # don't neglect correlations (different to the script)
#    if((sigma2_m + sigma2_c)[0]<0):    
#        print 'sigma2_m=', sigma2_m
#        print 'sigma2_c=', sigma2_c 
    sigma2 = np.sqrt(sigma2_m + sigma2_c + params.sigma_noise_I**2)
    return mu, sigma2
    
def m_derivative(t, m_small, parameters):
    """Calculates the derivative of m depending on the current values of m and c. One can chose between the computation
    in linear approximation in h0 and the exact expression."""
    m0        = parameters[0]   
    c0        = parameters[1:3]   
    S0        = parameters[3]   
    m_x       = parameters[4]
    h0        = parameters[5]
    omega     = parameters[6]
#    M_ana     = parameters[7]
    m_linear  = parameters[8] 
#    c_linear  = parameters[9] 
#    quadratic = parameters[10]
    c         = parameters[11:13]

    if(m_linear ==1):
        ######################################################
        ### Linear version: ##################################
        ######################################################    
        #m=np.hstack((m_small,m_x))
        mu0, sigma0 = mu_sigma_from_m_corr(np.hstack((m0,m_x)), c0)
        # Exactly copied from the "theory"-script:     
        m_der = -m_small +  m0 + np.dot(((params.J_array*params.K_array).T * S0).T, np.hstack((m_small-m0,0.)))  + np.dot(np.diag(np.array([S0[0],S0[1]])), h0 * np.sin(omega * t) * np.array([1.,1.]))
        # Now, add the part neglected in the lecture notes:
        m_der = m_der + np.dot(np.dot(np.dot(np.diag(S0*(params.theta - mu0)/(2*sigma0**2)),params.K_array * params.J_array *params.J_array),np.identity(len(m0)+1)-2.*np.diag(np.hstack((m0,m_x)))),np.hstack((m_small-m0,0.)))        
    else:
        ######################################################
        ### Non-linear version: ##############################
        #####################################################
        mu, sigma = mu_sigma_from_m_corr(np.hstack((m_small,m_x)), c)
        mu        = mu + h0 * np.sin(omega *t)
        m_der     = -m_small+phi(mu,sigma)    
        
    return m_der/params.tau_m

def deriv_c(t,c,parameters):
        """Calculates the derivative of c in a form of a vector in R⁵ in depending on the current values of m and c. One can chose between the computation
        in linear approximation in h0 and the exact expression."""
        m0        = parameters[0]   
        c0        = parameters[1:3]   
        S0        = parameters[3]   
        m_x       = parameters[4]
        h0        = parameters[5]
        omega     = parameters[6]
#        M_ana     = parameters[7]
#        m_linear  = parameters[8] 
        c_linear  = parameters[9] 
        quadratic = parameters[10]   
        m_small   = parameters[11]
        #print 'len(parameters)=', len(parameters)
        
        
#        print 'm0=', m0
#        print 'c0=', c0
#        print 'S0=', S0
#        print 'm_x=', params.m_x
#        print 'h0=', h0
#       
#        print 'c_linear=', c_linear    
#        print 'quadratic=', quadratic    
        #print 'm_small=', m_small
        
        m0        = np.hstack((m0, m_x))
#        print 'm0=', m0
        m         = np.hstack((m_small,m_x))
        a         = m  * (1.- m)
        a0        = m0 * (1.- m0)
#        For convenient calculations, we need c, c0, a and a0 to be in the form of matrices
        c_matrix = np.array([ [ c[II], c[IX] ],
                              [ c[IX], 0.    ] ])  
        c0_matrix = np.array([ [ c0[II], c0[IX] ],
                               [ c0[IX], 0.    ] ])   

        a0_diag = np.diag(a0/np.array([params.N_I,params.N_X]))
        a_diag  = np.diag(a/np.array([params.N_I,params.N_X]))
        
        if(c_linear == 1):
            ######################################################################
            ### Linear case: #####################################################
            ######################################################################
            mu0, sigma0 = mu_sigma_from_m_corr(m0, c0)
            
            Z  = np.zeros((2,2), dtype=float)        
            V  = np.zeros((2,2), dtype=float)        
            W0 = np.zeros((2,2), dtype=float)        
     
            #Z[0:2,:]= self.J*self.K
            Z[0:1,:]= params.J_array*params.K_array +np.dot(np.dot(np.diag(((params.theta-mu0)/sigma0 - sigma0/(params.theta - mu0)) * (1/(2*sigma0)) ), params.K_array*params.J_array*params.J_array), np.diag(1.-2.*m0))
            Z[1,:]  = np.zeros(2)        
            
            V[0:1,:] = ((params.J_array*params.K_array).T *(params.theta-mu0)/sigma0**2 * S0).T    
            V[1, :]  = np.zeros(2)           
                    
            W0[0:1,:] = ((params.J_array*params.K_array).T * S0).T    
            W0[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0
            
    
            if(quadratic == 1):
                Vc_plus_a = np.dot(V,a_diag + c_matrix)
            else:
                Vc_plus_a = np.dot(V,a0_diag + c0_matrix)
            
    #       inhomogeneous_non_sym = np.dot(np.diag(np.dot(Z,np.imag(M_ana * np.exp(1j * omega * t)))), Vc_plus_a) + np.dot(W0,np.dot(np.diag((1.-2.*m0))/np.array([params.N_E, params.N_I, params.N_X]) ,np.imag(M_ana * np.exp(1j * omega * t)))) +  Vc_plus_a * h0 * np.sin(omega *t)
    #       The terminology is a bit missleading: "inhomogeneous" means here the corrections for W coming from the linear order of S and "homogeneous" takes into account the variation of c and a            
            inhomogeneous_non_sym = np.dot(np.diag(np.dot(Z,m-m0)), Vc_plus_a)  +  Vc_plus_a * h0 * np.sin(omega *t) 
            inhomogeneous_natural = inhomogeneous_non_sym + np.transpose(inhomogeneous_non_sym)                      # Make the right side symmetrical
            
            homogeneous_non_sym = -(c_matrix - c0_matrix) + np.dot(W0,c_matrix - c0_matrix) + np.dot(W0,np.dot(np.diag((1.-2.*m0))/np.array([params.N_E, params.N_I, params.N_X]),np.diag(m-m0)))
            #homogeneous_non_sym = -(c_matrix - c0_matrix) + np.dot(W0,c_matrix - c0_matrix) + np.dot(W0,a_diag - a0_diag)
            homogeneous_natural = homogeneous_non_sym + np.transpose(homogeneous_non_sym) 
            
            c_deriv_matrix = inhomogeneous_natural + homogeneous_natural
            #c_deriv_matrix  = homogeneous_natural
        else:
            ######################################################################
            ### Non-linear case: #################################################
            ######################################################################
#            mu0, sigma0 = mu_sigma_from_m_corr(m0, c0)
#            
#            Z= np.zeros((3,3), dtype=float)        
#            W0= np.zeros((3,3), dtype=float)        
#     
#            #Z[0:2,:]= self.J*self.K
#            Z[0:2,:]= params.J_array*params.K_array +np.dot(np.dot(np.diag(((params.theta-mu0)/sigma0 - sigma0/(params.theta - mu0)) * (1/(2*sigma0)) ), params.K_array*params.J_array*params.J_array), np.diag(1.-2.*m0))
#            Z[2,:]  = np.zeros(3)        
#                    
#            W0[0:2,:] = ((params.J_array*params.K_array).T * S0).T    
#            W0[2, :]  = np.zeros(3)       #W_XE = W_XI = W_XX = 0
#            
#            S_lin_correcture = np.dot(np.dot(np.diag(S0 * (params.theta-mu0)/sigma0**2),Z[0:2]),m-m0)[:2] + S0 * (params.theta-mu0)/sigma0**2 * h0 * np.sin(omega * t)
#            W_lin_correcture = np.zeros((3,3),dtype=float)
#            W_lin_correcture[0:2,:] = ((params.J_array*params.K_array).T * S_lin_correcture).T             
            ###############################################################################################
            ### This is the case from above "inhomogeneous + homogeneous" in the "non-linear-notation": ###
            ###############################################################################################
    #        c_deriv_matrix = -2. * (c_matrix - c0_matrix) + np.dot(W0,c_matrix+a_diag - (c0_matrix + a0_diag))  + np.transpose(np.dot(W0,c_matrix + a_diag - (c0_matrix + a0_diag))) + np.dot(W_lin_correcture,c_matrix+a_diag)  + np.transpose(np.dot(W_lin_correcture,c_matrix + a_diag))
#            c_deriv_matrix = -2. * c_matrix + np.dot(W0 + W_lin_correcture,c_matrix+a_diag)  + np.transpose(np.dot(W0 + W_lin_correcture,c_matrix + a_diag))
    #        c_deriv_matrix = -2. * c_matrix + np.dot(W0,c_matrix+a_diag)  + np.transpose(np.dot(W0,c_matrix + a_diag))+ np.dot( W_lin_correcture,c_matrix+a_diag)  + np.transpose(np.dot( W_lin_correcture,c_matrix + a_diag))
            ################################################################################################
            ### This is the case from above "homogeneous" in the "non-linear-notation": ####################
            ################################################################################################        
    #        c_deriv_matrix = -2. * (c_matrix - c0_matrix) + np.dot(W0,c_matrix+a_diag - (c0_matrix + a0_diag))  + np.transpose(np.dot(W0,c_matrix + a_diag - (c0_matrix + a0_diag)))
            ################################################################################################
            ### The "full" (or "correct") case: ############################################################
            ################################################################################################        
            mu, sigma   = mu_sigma_from_m_corr(m,c)
            mu          = mu + h0 * np.sin(omega *t) 
            S           = susceptibility_corr(mu, sigma)            
            
            W = np.zeros((2,2), dtype=float)              
            W[0:1,:] = ((params.J_array*params.K_array).T * S).T    
            W[1, :]  = np.zeros(2)       #W_XE = W_XI = W_XX = 0
            
            c_deriv_matrix = -2. * c_matrix + np.dot(W,c_matrix+a_diag)  + np.transpose(np.dot(W,c_matrix + a_diag))
            #######################################################################
        c_deriv = np.zeros(2, dtype=complex)        
        c_deriv[II] = c_deriv_matrix[I,I]
        c_deriv[IX] = c_deriv_matrix[I,X]
        
        return c_deriv/params.tau_m

        
def deriv_m_and_c(t,m_and_c,parameters):
    """Calculates the derivative of the compound vector m_and_c unsing the functions m_derivative_lin and deriv_c""" 
    #print 'm_and_c=', m_and_c 
    m = m_and_c[:1]
    c = m_and_c[1:]
    #print "m in deriv_m_and_c gleich", m
    #print 'In deriv_m_and_c len(parameters)=', len(parameters)
    m_deriv = m_derivative(t, m, np.hstack((parameters,c)))
    c_deriv = deriv_c (t, c, np.hstack((parameters,m)))
    
    return np.hstack((m_deriv,c_deriv))

####################################################################################
### Calculate the mean activities and correlations for different values of omega ###
####################################################################################
def integrate_m_c_ode(t0,t1,dt, m_linear, c_linear, quadratic, h0 = params.h0_default, omega = params.omega_default):
    """Takes the start and ending times t0 and t1, the step size dt and the parameter omega and calculates the time course of the mean activity of a binary EI-network with random updates 
    and sinusodial perturbation with frequency omega in the inputs in mean-field approximation using the method scipy.integrate.ode. The solution of the stationary solution (without perturbation)
    is chosen as initial conditions. If m_linear == 1 or c_linear == 1, for the derivatives of the mean activities or the correlations respectively the linear approximations will be used.
    If additionally quadratic == 1 for the correlations the quadratic terms will be added to the linear ones."""
    N_t = int((t1-t0)/dt)
    N_transient      = int(10 * params.tau_m/dt)  
    #N_per_period = (1000. * 2. * np.pi/omega) / dt
    omega_kHz = omega/1000.    
    
    if(omega!=0):
        T = 2* np.pi/omega_kHz
    else:
        T = 100*dt
    print 'T=', T
    print 'dt=', params.dt_samp
    N_per_period = int(T/dt)
    print 'N_per_period = ', N_per_period
    N_number_periods = int((N_t-N_transient)/N_per_period)-1
    print 'N_number_periods=', N_number_periods
    N_eff=N_per_period * N_number_periods
    print 'N_eff + N_transient=', N_eff + N_transient        
    N_transient = (int(N_transient/N_per_period)+1.) * N_per_period
    print 'N_transient=', N_transient
    print 'omega/(2*np.pi)=', omega/(2*np.pi)
                        
    ginz= ginzburg(np.array([params.N_I,params.N_X]), params.J_array, params.K_array, params.m_x, params.theta, sigma_noise = np.array([params.sigma_noise_I]), omega = omega)
        
    #m_start = np.array([0.17, 0.08])
    m_start = params.m_exp
    m0, c0  = ginz.solve_self_consistently_corr(m_start)
    print 'm0=', m0
    print 'c0=', c0
        
    S0 = ginz.susceptibility_corr(m0,c0)
        
    parameters = np.zeros(11,dtype=complex)
    parameters[0]   = m0
    parameters[1:3] = c0
    parameters[3]   = S0
    parameters[4]   = params.m_x
    parameters[5]   = h0
    parameters[6]   = omega_kHz
    #parameters[7]  = M_ana
    parameters[8]   = m_linear
    parameters[9]   = c_linear
    parameters[10]  = quadratic
    
    
    r_neu = ode(deriv_m_and_c).set_integrator('Isoda', method='bdf',with_jacobian=False)
    r_neu.set_initial_value(np.hstack((m0,c0)),t0).set_f_params(parameters)
    t_list= np.zeros(N_t+2, dtype=float)
        
    #m_and_c_list = np.zeros((7,N_t+2),dtype=float)
    m_num_list = np.zeros((1,N_t+2),dtype=float)
    c_num_list = np.zeros((2,N_t+2),dtype=float)
        
    jj=0
    while r_neu.successful() and r_neu.t < t1:
        r_neu.integrate(r_neu.t+dt)
        t_list[jj] = r_neu.t
            
        m_num_list[:,jj] = r_neu.y[:1]
        c_num_list[:,jj] = r_neu.y[1:]
        jj=jj+1
    
    return t_list, m_num_list, c_num_list, m0, c0